/*
 ## Cypress FX3 Core Library Header (CyFx3_uvc.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

#ifndef _INCLUDED_CyFx3_UVC_H_
#define _INCLUDED_CyFx3_UVC_H_

#include "cyu3types.h"
#include "cyu3usbconst.h"
#include "cyu3os.h"
#include "cyu3utils.h"
#include "cyu3mipicsi.h"
#include "cyu3externcstart.h"
#include "cyu3tx.h"

#ifdef UVC_HEADER
/* UVC Header*/
static uint8_t glUVCHeader[12] =
{
  0x0C,                               /* Header Length */
  0x8C,                               /* Bit field header field */
  0x00, 0x00, 0x00, 0x00,             /* Presentation time stamp field */
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00  /* Source clock reference field */
};
#endif

/* Uncomment the following line to Enable BULK UVC data transfer */
//#define USE_ISOCHRONOUS_ENDPOINT

/* The following line indicates the time period after which ISP will enter low power mode.
 * This timeout is required because the FX3 device receives an EP_CLEAR_FEATURE even when a resolution is changed in host application.
 * The time-out will ensure that ISP does not go into low power mode every time it receives EP_CLEAR_FEATURE requests */
#define ISP_LOW_POWER_ENTRY_TIMEOUT 1000

/* Uncomment the following line to disable entering into Low power mode */
//#define ENABLE_ISP_LOW_POWER_MODE

/* Uncomment the following line to provide verbose debug logging-level 1
 * Prints the Get Cur Probe, Set Cur Probe and Set Cur Commit events for Video
 * and Still events. */
#define FX3_DEBUG_ENABLE_LEVEL1

/* Uncomment the following line to provide verbose debug logging.- level 2
 * Prints all USB events, Start/Stop of Application code */
//#define FX3_DEBUG_ENABLE_LEVEL2

/*To enable debug prints at ISP register level access, uncomment the line below*/
#define ISP_DEBUG

/*To enable the UART Debug Prints for Video Control Interface uncomment the line below.
 * NOTE: Enabling this will generate a lot of traffic over UART*/
#define ISP_VIDEO_CONTROL_DEBUG

/* Uncomment the line below to print frame information like Producer count,
 * Consumer count, Partial buffer size, Frame count, Frame size and FPS*/
#define PRINT_FRAME_INFO
#define FPS 1

/* Uncomment the following line to run as UVC 1.5 device or as UVC 1.0 device.
 * Default is UVC 1.1 device */
#define FX3_UVC_1_5_SUPPORT
//#define FX3_UVC_1_0_SUPPORT

/*Uncomment the line below to enable HID interface*/
#define ENABLE_HID_INTERFACE

/* Uncomment this to enable a timer that aborts an ongoing frame and restarts streaming when the transfer is
 * stalled. Default setting is timer enabled. */
#define FRAME_TIMER_ENABLE

/*To enable the UVC Video Control interface, uncomment the below macro */
#define VIDEO_CONTROL_INTERFACE

/* Macros to store ASCII values for entering/Exiting test modes*/
#define ASCII_VALUE_FOR_T               (84)
#define ASCII_VALUE_FOR_M               (77)
#define ASCII_VALUE_FOR_N               (78)

/* SPI flash addresses to erase data and write test mode registers */
#define SPI_FLASH_ERASE_ADDRESS         (0)        /*This is the address where first sector starts*/
#define SPI_TEST_MODE_VAR_ADDRESS       (192*1024) /*This is the place where fourth sector starts*/

/* Macros to handle MS OS descriptor requests to enable selective suspend for HID based UVC device */
#define CY_U3P_USB_MS_STRING_DSCR             0xEE
#define CY_U3P_USB_GET_MS_EXT_PROP_DSCR       0xC1
#define CY_U3P_USB_MS_EXT_PROP_TYPE           0x05
#define CY_U3P_USB_MS_EXT_PROP_HDR_SIZE       0x0A
#define CY_U3P_USB_MS_EXT_PROP_DSCR_SIZE      0x4C

/*To enumerate as UVC device uncomment the below macro. if the below macro is disabled, FX3 enumerates as Vendor class device*/
#define UVC_APPLICATION

/*Uncomment the line below to enable Still Captures*/
#define STILL_CAPTURE_ENABLE

/*FX3 Firmware version is being maintained in the macros below*/
#define FX3_FW_VERSION_MAJOR 0
#define FX3_FW_VERSION_MINOR 13
#define FX3_FW_BUILD_MONTH   10
#define FX3_FW_BUILD_DAY     17
#define FX3_FW_BUILD_YEAR    18

#define UVC_APP_THREAD_STACK                    (0x1000)        /* Thread stack size */
#define UVC_APP_THREAD_PRIORITY                 (8)             /* Thread priority */

#ifdef FX3_ERROR_THREAD_ENABLE
#define UVC_MIPI_ERROR_THREAD_STACK             (0x1000)        /* Thread stack size */
#define UVC_MIPI_ERROR_THREAD_PRIORITY          (15)            /* Thread priority */
#endif

/* Macros used for ISOC transfer. */
#define FX3_EP_ISO_VIDEO_SS_BURST               (0x0E)          /* BURST setting = 15 - 1*/
#define FX3_EP_ISO_VIDEO_SS_MULT                (0x02)          /* Mult setting = 2 */
#define FX3_UVC_ISO_PAYLOAD_SIZE                ((FX3_EP_ISO_VIDEO_SS_BURST + 1) * (FX3_EP_ISO_VIDEO_SS_MULT + 1) * 1024)
#define FX3_UVC_ISO_PAYLOAD_SIZE_L              CY_U3P_GET_LSB(FX3_UVC_ISO_PAYLOAD_SIZE)
#define FX3_UVC_ISO_PAYLOAD_SIZE_H              CY_U3P_GET_MSB(FX3_UVC_ISO_PAYLOAD_SIZE)

/* Wait for some time after sending a NAK or after flow control of Endpoint*/
#define FX3_EP_BUSY_WAIT                        (125)

/**< Initial value of early outputs from the FX3 GPIF state machine. */
#define ALPHA_FX3_START_SCK0                    0x0
/**< Initial value of early outputs from the FX3 GPIF state machine. */
#define ALPHA_FX3_START_SCK1                    0x0

/* Endpoint definition for UVC application */
#define FX3_EP_CONTROL_STATUS_1                 (0x82)          /* EP 2 IN */
#define FX3_EP_BULK_VIDEO_1                     (0x83)          /* EP 3 IN */

#define FX3_EP_HID_PROD		                    (0x08)          /* EP 8 OUT */
#define FX3_EP_HID_CONS		                    (0x89)          /* EP 9 IN */

/* FX3 UVC Stream Interfaces  */
#define FX3_UVC_STREAM_INTERFACE_1              (0x01)

#define FX3_UVC_SOCKET_COUNT                    (2)                         /* Socket Count for Manual Many to One
                                                                               DMA Channel */
#define FX3_EP_VIDEO_CONS_SOCKET_2              (CY_U3P_UIB_SOCKET_CONS_7)  /* Consumer socket 7 */
#define FX3_EP_VIDEO_CONS_SOCKET_1              (CY_U3P_UIB_SOCKET_CONS_3)  /* Consumer socket 3 */
#define FX3_EP_VIDEO_CONS_SOCKET_3              (CY_U3P_UIB_SOCKET_CONS_5)  /* Consumer socket 5 */
#define FX3_EP_STATUS_CONS_SOCKET_1             (CY_U3P_UIB_SOCKET_CONS_2)  /* Consumer socket 2 */
#define FX3_EP_STATUS_CONS_SOCKET_2             (CY_U3P_UIB_SOCKET_CONS_6)  /* Consumer socket 6 */
#define FX3_EP_STATUS_CONS_SOCKET_3             (CY_U3P_UIB_SOCKET_CONS_4)  /* Consumer socket 4 */
#define FX3_PRODUCER_PPORT_SOCKET_0             (CY_U3P_PIB_SOCKET_0)       /* P-port Socket 0 is producer */
#define FX3_PRODUCER_PPORT_SOCKET_1             (CY_U3P_PIB_SOCKET_1)       /* P-port Socket 0 is producer */

/* Endpoint Packet Sizes */
#define FX3_EP_BULK_FULL_SPEED_PKT_SIZE         (0x40)  /* Full Speed endpoint packet Size */
#define FX3_EP_BULK_HIGH_SPEED_PKT_SIZE         (0x200) /* High Speed endpoint packet Size */
#define FX3_EP_INTR_PACKET_SIZE                 (0x40)  /* Interrupt endpoint packet Size */

/* Endpoint Burst Lengths */
#define FX3_EP_INTR_BURST_LEN                   (1)     /* Interrupt Endpoint Burst Length */
#define FX3_EP_BULK_SUPER_SPEED_BURST_LEN       (16)    /* Super Speed endpoint burst length */
#define FX3_EP_BULK_HIGH_SPEED_BURST_LEN        (1)     /* High Speed endpoint burst length */
#define FX3_EP_BULK_FULL_SPEED_BURST_LEN        (1)     /* High Speed endpoint burst length */

/* UVC descriptor types */
#define FX3_INTRFC_ASSN_DESCR                   (11)    /* Interface association descriptor type. */
#define FX3_CS_INTRFC_DESCR                     (0x24)  /* Class Specific Interface Descriptor type: CS_INTERFACE */
#define FX3_EP_BULK_VIDEO_PKT_SIZE              (0x400) /* UVC video streaming endpoint packet Size */
#define FX3_EP_HS_BULK_VIDEO_PKT_SIZE			(0x200)
/* UVC Buffer Parameters*/


/*FX3_UVC_DATA_BUF_SIZE value of 0x4000 is compatible with 16-bit GPIF configuration;
* When using Streamer for getting the video data FX3_UVC_DATA_BUF_SIZE has to be a
* multiple of End point max packet size (1024), hence 0x4000 was chosen for FX3_UVC_DATA_BUF_SIZE;
* CAUTION: 0x6000 can be used and is compatible with both 16-bit/24-bit GPIF, but cannot be used for 720p;
* 720p frame size is exact multiple of 0x6000, hence the frame end cannot be
* detected with this value for 720p, since partial buffer does not exist in this case;

* When using UVC, 0x5FD0 for FX3_UVC_DATA_BUF_SIZE will work for 16-bit/24-bit GPIF configuration and for all resolutions
* with 4 as FX3_UVC_STREAM_BUF_COUNT;

*/

/* UVC DMA Buffer Size = Should be divisible by 48 (16 bytes for Buffer allocation and 3 for GPIF load)
 * and shouldn't be multiple of frame size */
#define FX3_UVC_DATA_BUF_SIZE                   (48*383)
#define FX3_UVC_PROD_HEADER                     (12)       /* UVC DMA Buffer Header Size */
#define FX3_UVC_PROD_FOOTER                     (4)        /* UVC DMA Buffer Footer Size */

/* UVC Buffer size - Will map to bulk Transaction size */
#define FX3_UVC_STREAM_BUF_SIZE                 (42*1024) //(FX3_UVC_DATA_BUF_SIZE + FX3_UVC_PROD_HEADER + FX3_UVC_PROD_FOOTER) //moved to UVC.c

/* UVC Buffer count */
#define FX3_UVC_STREAM_BUF_COUNT                (2)

/* Low byte - UVC video streaming endpoint packet size */
#define FX3_EP_BULK_VIDEO_PKT_SIZE_L            CY_U3P_GET_LSB(FX3_EP_BULK_VIDEO_PKT_SIZE)
#define FX3_EP_HS_BULK_VIDEO_PKT_SIZE_L         CY_U3P_GET_LSB(FX3_EP_HS_BULK_VIDEO_PKT_SIZE)

/* High byte - UVC video streaming endpoint packet size */
#define FX3_EP_BULK_VIDEO_PKT_SIZE_H            CY_U3P_GET_MSB(FX3_EP_BULK_VIDEO_PKT_SIZE)
#define FX3_EP_HS_BULK_VIDEO_PKT_SIZE_H            CY_U3P_GET_MSB(FX3_EP_HS_BULK_VIDEO_PKT_SIZE)


#define FX3_UVC_HEADER_LENGTH                   (FX3_UVC_PROD_HEADER)           /* Maximum number of header bytes in UVC */
#define FX3_UVC_HEADER_DEFAULT_BFH              (0x8C)                          /* Default BFH(Bit Field Header) for the UVC Header */

#ifdef FX3_UVC_1_0_SUPPORT
#define FX3_UVC_MAX_PROBE_SETTING               (26)            /* UVC 1.0 Maximum number of bytes in Probe Control */
#define FX3_UVC_MAX_PROBE_SETTING_ALIGNED       (32)            /* Maximum number of bytes in Probe Control aligned to 32 byte */
#else
#ifdef FX3_UVC_1_5_SUPPORT
#define FX3_UVC_MAX_PROBE_SETTING               (48)            /* UVC 1.1 Maximum number of bytes in Probe Control */
#define FX3_UVC_MAX_PROBE_SETTING_ALIGNED       (64)            /* Maximum number of bytes in Probe Control aligned to 32 byte */
#else
#define FX3_UVC_MAX_PROBE_SETTING               (34)            /* UVC 1.1 Maximum number of bytes in Probe Control */
#define FX3_UVC_MAX_PROBE_SETTING_ALIGNED       (64)            /* Maximum number of bytes in Probe Control aligned to 32 byte */
#endif
#endif

#define FX3_UVC_HEADER_FRAME                    (0)                /* Normal frame indication */
#define FX3_UVC_HEADER_EOF                      (uint8_t)(1 << 1)  /* End of frame indication */
#define FX3_UVC_HEADER_FRAME_ID                 (uint8_t)(1 << 0)  /* Frame ID toggle bit */

#define FX3_UVC_VC_STATUS_INT_HEADER            (5)                /* UVC VC status Interrupt EP Buffer Header Size */
#define FX3_UVC_VC_STATUS_INT_PAYLOAD           (15)               /* This is same as the GET_CUR size of the XU */

#define FX3_USB_UVC_SET_REQ_TYPE                (uint8_t)(0x21)    /* UVC interface SET request type */
#define FX3_USB_UVC_GET_REQ_TYPE                (uint8_t)(0xA1)    /* UVC Interface GET request type */
#define FX3_USB_UVC_GET_CUR_REQ                 (uint8_t)(0x81)    /* UVC GET_CUR request */
#define FX3_USB_UVC_SET_CUR_REQ                 (uint8_t)(0x01)    /* UVC SET_CUR request */
#define FX3_USB_UVC_GET_MIN_REQ                 (uint8_t)(0x82)    /* UVC GET_MIN Request */
#define FX3_USB_UVC_GET_MAX_REQ                 (uint8_t)(0x83)    /* UVC GET_MAX Request */
#define FX3_USB_UVC_GET_RES_REQ                 (uint8_t)(0x84)    /* UVC GET_RES Request */
#define FX3_USB_UVC_GET_LEN_REQ                 (uint8_t)(0x85)    /* UVC GET_LEN Request */
#define FX3_USB_UVC_GET_INFO_REQ                (uint8_t)(0x86)    /* UVC GET_INFO Request */
#define FX3_USB_UVC_GET_DEF_REQ                 (uint8_t)(0x87)    /* UVC GET_DEF Request */

#define FX3_UVC_VS_PROBE_CONTROL                (0x0100)           /* Video Stream Probe Control Request */
#define FX3_UVC_VS_COMMIT_CONTROL               (0x0200)           /* Video Stream Commit Control Request */
#define FX3_UVC_VC_REQUEST_ERROR_CODE_CONTROL   (0x0200)           /* Request Control Error Code*/
#define FX3_UVC_ERROR_INVALID_CONTROL           (0x06)             /* Error indicating invalid control requested*/
#define FX3_UVC_STREAM_INTERFACE                (0x01)             /* Streaming Interface : Alternate setting 1 */
#define FX3_UVC_CONTROL_INTERFACE_1             (0x00)             /* Control Interface: Alternate Setting 0 */

#ifdef VIDEO_CONTROL_INTERFACE
#define FX3_UVC_VC_DESCRIPTOR_UNDEFINED         (0x00)             /* Control Interface: Undefined ID */
#define FX3_UVC_VC_CAMERA_TERMINAL_ID           (0x01)             /* Control Interface: Camera Terminal ID */
#define FX3_UVC_VC_PROCESSING_UNIT_ID           (0x02)             /* Control Interface: Processing Unit ID */
#define FX3_UVC_VC_EXTENSION_UNIT_ID            (0x03)             /* Control Interface: Extension Unit */

/*****Undefined Terminal Controls******/
#define VC_REQUEST_ERROR_CODE_CONTROL           (0x02)

/* Error Codes */
#define VC_ERROR_CODE_NO_ERROR                  (0x00)
#define VC_ERROR_CODE_NOT_READY                 (0x01)
#define VC_ERROR_CODE_WRONG_STATE               (0x02)
#define VC_ERROR_CODE_POWER                     (0x03)
#define VC_ERROR_CODE_OUT_OF_RANGE              (0x04)
#define VC_ERROR_CODE_INVALID_UNIT              (0x05)
#define VC_ERROR_CODE_INVALID_CONTROL           (0x06)
#define VC_ERROR_CODE_INVALID_REQUEST           (0x07)
#define VC_ERROR_CODE_INVALID_VAL_IN_RANGE      (0x08)
#define VC_ERROR_CODE_UNKNOWN                   (0xFF)

/*****Camera Terminal Controls******/
#define CT_CONTROL_UNDEFINED                    0x00
#define CT_SCANNING_MODE_CONTROL                0x01
#define CT_AE_MODE_CONTROL                      0x02
#define CT_AE_PRIORITY_CONTROL                  0x03
#define CT_EXPOSURE_TIME_ABSOLUTE_CONTROL       0x04
#define CT_EXPOSURE_TIME_RELATIVE_CONTROL       0x05
#define CT_FOCUS_ABSOLUTE_CONTROL               0x06
#define CT_FOCUS_RELATIVE_CONTROL               0x07
#define CT_FOCUS_AUTO_CONTROL                   0x08

/****Processing Unit Controls*********/
#define PU_CONTROL_UNDEFINED                    0x00
#define PU_BACKLIGHT_COMPENSATION_CONTROL       0x01
#define PU_BRIGHTNESS_CONTROL                   0x02
#define PU_CONTRAST_CONTROL                     0x03
#define PU_SATURATION_CONTROL                   0x07
#define PU_SHARPNESS_CONTROL                    0x08
#define PU_GAMMA_CONTROL                        0x09
#define PU_WHITE_BALANCE_TEMPERATURE_CONTROL    0x0A
#define PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL 0x0B
#define PU_WHITE_BALANCE_COMPONENT_CONTROL      0x0C
#define PU_WHITE_BALANCE_COMPONENT_AUTO_CONTROL 0x0D
#endif

#define FX3_UVC_VS_PROBE_CONTROL                (0x0100)           /* Control selector for VS_PROBE_CONTROL. */
#define FX3_UVC_VS_COMMIT_CONTROL               (0x0200)           /* Control selector for VS_COMMIT_CONTROL. */
#define FX3_UVC_VS_STILL_PROBE_CONTROL          (0x0300)           /* Control selector for VS_STILL_PROBE_CONTROL. */
#define FX3_UVC_VS_STILL_COMMIT_CONTROL         (0x0400)           /* Control selector for VS_STILL_COMMIT_CONTROL. */
#define FX3_UVC_VS_STILL_IMAGE_TRIGGER_CONTROL  (0x0500)           /* Control selector for VS_STILL_IMAGE_TRIGGER_CONTROL. */

#define FX3_GPIF_SWITCH_TIMEOUT                 (2)                /* Timeout setting for the switch operation in GPIF clock cycles */
#define FX3_INVALID_GPIF_STATE                  (257)              /* Invalid state for use in CyU3PGpifSMSwitch calls */

/* Setup data field : Request */
#define CY_U3P_USB_REQUEST_MASK                 (0x0000FF00)
#define CY_U3P_USB_REQUEST_POS                  (8)


#define FX3_DMA_DBG_EVT							(1 << 10)
#define FX3_DMA_RESET_EVENT                     (1 << 0)           /* DMA Reset event */
#define FX3_USB_SUSP_EVENT_FLAG                 (1 << 1)           /* Event generated on a USB Suspend Request*/
#define FX3_HID_PROD_EVENT                      (1 << 2)           /* Event generated for Async Video control event*/
#define FX3_USB_ASYNC_VC_EVENT                  (1 << 3)           /* Event generated due to HID USB data Prod event*/
#define FX3_USB_VC_AE_STATUS_CHK_EVENT          (1 << 4)           /* Event generated due to AE SET_CUR, to check Status from ISP*/
#define FX3_USB_VC_AF_STATUS_CHK_EVENT          (1 << 5)           /* Event generated due to AF SET_CUR, to check Status from ISP*/
#define FX3_USB_VC_AWB_STATUS_CHK_EVENT         (1 << 6)           /* Event generated due to AWB SET_CUR, to check Status from ISP*/

#ifdef STILL_CAPTURE_ENABLE
#define FX3_UVC_MAX_STILL_PROBE_SETTING         (11)               /* Maximum number of bytes in Still Probe Control */
#define FX3_UVC_HEADER_STI                      (uint8_t)(1 << 5)  /* Still Image Indication in the UVC Header*/
#define FX3_STILL_IMAGE_EVENT                   (1 << 7)           /*Event used to Still Image Capture*/
#endif

/* Variable to store FX3 FW version */
extern uint8_t glFx3FwVer[5];



/* Extern definitions of the USB Enumeration constant arrays used for the Application */
extern const uint8_t CyFx3USB20DeviceDscr[];
extern const uint8_t CyFx3USB30DeviceDscr[];
extern const uint8_t CyFx3USBDeviceQualDscr[];
extern const uint8_t CyFx3USBFSConfigDscr[];
extern uint8_t CyFx3USBHSConfigDscr[];
extern const uint8_t CyFx3USBBOSDscr[];
extern uint8_t CyFx3USBSSConfigDscr[];
extern uint8_t CyFx3USBSSTestModeConfigDscr[];
extern const uint8_t CyFx3USBStringLangIDDscr[];
extern const uint8_t CyFx3USBManufactureDscr[];
extern const uint8_t CyFx3USBProductDscr[];
extern const uint8_t CyFx3USBConfigSSDscr[];
extern const uint8_t CyFx3USBConfigHSDscr[];
extern const uint8_t CyFx3USBConfigFSDscr[];
extern const uint8_t CyFx3USBProductDescr[];
extern const uint8_t CyFx3USB_HID_Stringdesc[];
extern const uint8_t CyFx3UsbMsOSStringDscr[];
extern const uint8_t CyFx3UsbMsOSExtPropDscr[];

/* UVC Probe Control Setting */
extern uint8_t       glProbeCtrl[FX3_UVC_MAX_PROBE_SETTING];                /* Scratch/main buffer. */


#ifdef SHRINK_OVERALL_CODE_SIZE

/* Frame size can be retrieved from this array */
extern const uint32_t glCommonFrameSize[2][10];

#else
extern uint8_t glProbeCtrl_1920x1080_60fps[FX3_UVC_MAX_PROBE_SETTING];
extern uint8_t glProbeCtrl_1280x720_60fps[FX3_UVC_MAX_PROBE_SETTING];
extern uint8_t glProbeCtrl_640x640_60fps[FX3_UVC_MAX_PROBE_SETTING];
extern uint8_t glProbeCtrl_640x480_60fps[FX3_UVC_MAX_PROBE_SETTING];
extern uint8_t glProbeCtrl_640x360_60fps[FX3_UVC_MAX_PROBE_SETTING];

extern uint8_t glProbeCtrl_1280x720_15fps[FX3_UVC_MAX_PROBE_SETTING];
extern uint8_t glProbeCtrl_640x480_60fps_HS[FX3_UVC_MAX_PROBE_SETTING];

#endif

#define GPIF_BUS_WIDTH CY_U3P_GPIF_BUS_16 /*Macro for GPIF Bus Width*/

#define UVC_STREAM

extern uint8_t LED_Status;
extern uint8_t EnableGpioInterrupt;


#define     SENSOR_1P8_EN_GPIO23 23      /* Sensor 1P8 Power Supply enable pin GPIO 23 */
#define     TYPE_C_ENABLED
#define     SS_MUX_SEL_GPIO    	60      /* Type-C Mux Selection pin GPIO 22 */
#define		DEBUG_TRACE_LEVEL 	 6

#define ECP5_5G_ID		(0x41112043)
#define READ_ID_CMD		(0xE0)
#define REFRESH_CMD		(0x79)
#define WRITE_EN_CMD	(0xC6)
#define WRITE_BITS_CMD	(0x7A)
#define READ_STATUS_CMD	(0x3C)
#define WRITE_DIS_CMD	(0x26)
#define SRAM_ERASE		(0x0E)
#define INIT_ADDR		(0x46)

#define FPGA_SSN		(57)
#define FX3_LED			(51)
#define PROG_N_GPIO		(27)
#define INIT_N_GPIO		(26)
#define DONE_GPIO		(25)
#define MUX_SEL_GPIO	(60)
#define SW3_GPIO        (45)
#define SW2_GPIO        (50)
#define SLOE_GPIO       (19)
#define SLRD_GPIO       (20)
#define ISP_SPI_SS	(0xFF)

/* This function starts the video streaming application. It is called
 * when there is a SET_INTERFACE event for alternate interface 1
 * (in case of UVC over Isochronous Endpoint usage) or when a
 * COMMIT_CONTROL(SET_CUR) request is received (when using BULK only UVC).
 */
extern CyU3PReturnStatus_t CyFx3UvcAppStart(uint8_t EPNumber);

/* This function stops the video streaming. It is called from the USB event
 * handler, when there is a reset / disconnect or SET_INTERFACE for alternate
 * interface 0 in case of isochronous implementation or when a Clear Feature (Halt)
 * request is received (in case of bulk only implementation).
 */
extern void CyFx3UvcAppStop();

void CySendSpiCmd(uint8_t cmd);
void CyWriteStreamCmd(void);
void CyFx3UvcIoMatrixReInit(void);
void CyFx3UvcInitGpifSm(void);

#include "cyu3externcend.h"
#endif /* _INCLUDED_CyFx3_UVC_H_ */

