/*
 ## Cypress FX3 Firmware Example Source (cyFX3_uvcdscr.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2013-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *
 *  Created on: 30-June-2015
 *  Author: savj
 */

/* This file contains the USB enumeration descriptors for the FX3 Video Streaming application example.
 * The descriptor arrays must be 32 byte aligned if the D-cache is turned on. If the linker
 * used is not capable of supporting the aligned feature for this, then dynamically allocated
 * buffer must be used, and the descriptor must be loaded into it. */

#include "cyfx3_uvc.h"
#include "cyu3mipicsi.h"
#include "cyfx3_videostreaming.h"

#include "stddef.h"
#include "string.h"

/* Standard Device Descriptor for USB 3 */
const uint8_t CyFx3USB30DeviceDscr[] =
{
		0x12,                               /* Descriptor size */
		CY_U3P_USB_DEVICE_DESCR,            /* Device descriptor type */
		0x10, 0x03,                         /* USB 3.0 */
		0xEF,                               /* Device class */
		0x02,                               /* Device Sub-class */
		0x01,                               /* Device protocol */
		0x09,                               /* Maxpacket size for EP0 : 2^9 */
		0xB4, 0x04,                         /* Vendor ID */
		0x40, 0x00,                         /* Product ID */
		0x00, 0x01,                         /* Device release number */
		0x01,                               /* Manufacture string index */
		0x02,                               /* Product string index */
		0x00,                               /* Serial number string index */
		0x01                                /* Number of configurations *///
};

/* Standard Device Descriptor for USB 2 */
const uint8_t CyFx3USB20DeviceDscr[] =
{
		0x12,                               /* Descriptor size */
		CY_U3P_USB_DEVICE_DESCR,            /* Device descriptor type */
		0x10, 0x02,                         /* USB 2.1 */
		0xEF,                               /* Device class */
		0x02,                               /* Device sub-class */
		0x01,                               /* Device protocol */
		0x40,                               /* Maxpacket size for EP0 : 64 bytes */
		0xB4, 0x04,                         /* Vendor ID */
		0x40, 0x00,                         /* Product ID */
		0x00, 0x00,                         /* Device release number */
		0x01,                               /* Manufacture string index */
		0x02,                               /* Product string index */
		0x00,                               /* Serial number string index */
		0x01                                /* Number of configurations */
};

/* Binary Device Object Store (BOS) Descriptor */
const uint8_t CyFx3USBBOSDscr[] =
{
		0x05,                               /* Descriptor size */
		CY_U3P_BOS_DESCR,                   /* Device descriptor type */
		0x16, 0x00,                         /* Length of this descriptor and all sub descriptors */
		0x02,                               /* Number of device capability descriptors */

		/* USB 2.0 Extension */
		0x07,                               /* Descriptor size */
		CY_U3P_DEVICE_CAPB_DESCR,           /* Device capability type descriptor */
		CY_U3P_USB2_EXTN_CAPB_TYPE,         /* USB 2.1 extension capability type */
		0x02, 0x00, 0x00, 0x00,             /* Supported device level features - LPM support */

		/* SuperSpeed Device Capability */
		0x0A,                               /* Descriptor size */
		CY_U3P_DEVICE_CAPB_DESCR,           /* Device capability type descriptor */
		CY_U3P_SS_USB_CAPB_TYPE,            /* SuperSpeed device capability type */
		0x00,                               /* Supported device level features  */
		0x0E, 0x00,                         /* Speeds supported by the device : SS, HS and FS */
		0x03,                               /* Functionality support */
		0x00,                               /* U1 device exit latency */
		0x00, 0x00                          /* U2 device exit latency */
};

/* Standard Device Qualifier Descriptor */
const uint8_t CyFx3USBDeviceQualDscr[] =
{
		0x0A,                               /* descriptor size */
		CY_U3P_USB_DEVQUAL_DESCR,           /* Device qualifier descriptor type */
		0x00, 0x02,                         /* USB 2.0 */
		0xEF,                               /* Device class */
		0x02,                               /* Device sub-class */
		0x01,                               /* Device protocol */
		0x40,                               /* Maxpacket size for EP0 : 64 bytes */
		0x01,                               /* Number of configurations */
		0x00                                /* Reserved */
};

/* Standard Super Speed Configuration Descriptor */
uint8_t CyFx3USBSSConfigDscr[] =
{
		/* Configuration Descriptor*/
		0x09,                               /* Descriptor Size */
		CY_U3P_USB_CONFIG_DESCR,            /* Configuration Descriptor Type */
		SS_TOTAL_SIZE_CONFIG_DSCR_L,        /* Length of this descriptor and all sub descriptors - low Byte */
		SS_TOTAL_SIZE_CONFIG_DSCR_H,        /* Length of this descriptor and all sub descriptors - high Byte */
		0x02,                               /* Number of interfaces */
		0x01,                               /* Configuration number */
		0x03,                               /* Configuration string index */
		0x80,                               /* Config characteristics - Bus Powered */
		0x0C,                               /* Max power consumption of device (in 8mA unit) : 96mA */

		/****************************************************************************************************************************
		 *************************************** Video Streaming Interface - 1 (FRONT CAM) ******************************************
		 ****************************************************************************************************************************/

		/* Interface Association Descriptor */
		0x08,                               /* Descriptor Size */
		FX3_INTRFC_ASSN_DESCR,              /* Interface Association Descriptor Type */
		0x00,                               /* Interface number of the VideoControl interface
                                               that is associated with this function*/
		0x02,                               /* Number of contiguous Video interfaces that are
                                               associated with this function */
		0x0E,                               /* Video Interface Class Code: CC_VIDEO */
		0x03,                               /* Subclass code: SC_VIDEO_INTERFACE_COLLECTION*/
		0x00,                               /* Protocol: PC_PROTOCOL_UNDEFINED*/
		0x06,                               /* String Descriptor index for interface */

		/* Standard Video Control Interface Descriptor (Interface 0, Alternate Setting 0)*/
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface Descriptor type */
		0x00,                               /* Index of this Interface */
		0x00,                               /* Alternate setting number */
		0x01,                               /* Number of end points - 1 Interrupt Endpoint*/
		0x0E,                               /* Video Interface Class Code: CC_VIDEO  */
		0x01,                               /* Interface sub class: SC_VIDEOCONTROL */
		0x00,                               /* Interface protocol code: PC_PROTOCOL_UNDEFINED.*/
		0x06,                               /* Interface descriptor string index */

		/* Class specific VC Interface Header Descriptor */
		0x0D,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class Specific Interface Descriptor type: CS_INTERFACE */
		0x01,                               /* Descriptor Sub type: VC_HEADER */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x01,                         /* Revision of UVC class spec: 1.0 - Legacy version */
		0x50, 0x00,                         /* Total Size of class specific descriptors
                                        (till Output terminal) */
#else
#ifdef FX3_UVC_1_5_SUPPORT
		0x50, 0x01,                         /* Revision of UVC class spec: 1.1 - Minimum version required
                                        for USB Compliance. Not supported on Windows XP*/
#else
		0x10, 0x01,                         /* Revision of UVC class spec: 1.1 - Minimum version required
                                        for USB Compliance. Not supported on Windows XP*/
#endif
		0x51, 0x00,                         /* Total Size of class specific descriptors
                                        (till Output terminal) */
#endif
		0x00, 0x6C, 0xDC, 0x02,            /* Clock frequency : 48MHz(Deprecated) */
		0x01,                              /* Number of streaming interfaces */
		0x01,                              /*VideoStreaming interface 1 belongs to this
                                        VideoControl interface*/

		/* Input (Camera) Terminal Descriptor */
		0x12,                             /* Descriptor size */
		FX3_CS_INTRFC_DESCR,              /* CS_INTERFACE */
		0x02,                             /* VC_INPUT_TERMINAL subtype */
		0x01,                             /* ID of this input terminal */
		0x01, 0x02,                       /* ITT_CAMERA type. This terminal is a camera
                                             terminal representing the CCD sensor*/
		0x00,                             /* No association terminal */
		0x00,                             /* Unused */
		0x00, 0x00,                       /* No optical zoom supported */
		0x00, 0x00,                       /* No optical zoom supported */
		0x00, 0x00,                       /* No optical zoom supported */
		0x03,                             /* Size of controls field for this terminal : 3 bytes */
#ifndef VIDEO_CONTROL_INTERFACE
		0x00, 0x00, 0x00,                 /* No controls supported */
#else
		0x0A, 0x00, 0x00, //0x2A, 0x02, 0x32,
#endif

		/* Processing Unit Descriptor */
#ifndef FX3_UVC_1_0_SUPPORT
		0x0D,                               /* Descriptor size */
#else
		0x0C,                               /* Descriptor size */
#endif
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x05,                               /* Processing Unit Descriptor type: VC_PROCESSING_UNIT*/
		0x02,                               /* ID of this unit */
		0x01,                               /* Source ID: 1: Connected to input terminal */
		0x00, 0x40,                         /* Digital multiplier */
		0x03,                               /* Size of controls field for this terminal: 3 bytes */
#ifndef VIDEO_CONTROL_INTERFACE
		0x00, 0x00, 0x00,                   /* No controls supported */
#else
		0x6B, 0x10, 0x00,                   /* bmControl supported */
#endif
		0x00,                               /* String descriptor index: Not used */
#ifndef FX3_UVC_1_0_SUPPORT
		0x00,                               /* Analog Video Standards Supported: None */
#endif

		/* Extension Unit Descriptor */
		0x1C,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x06,                               /* Extension Unit Descriptor type */
		0x03,                               /* ID of this terminal */

		//static const GUID xuGuidIsRdk =
		//  { 0x2c7b9c4d, 0xccc2, 0x47da,{ 0x82, 0xf2, 0x7f, 0x7a, 0x26, 0x3a, 0x49, 0xc2 } };
		0x4D, 0x9C, 0x7B, 0x2C,             /* 16 byte GUID */
		0xC2, 0xCC, 0xDA, 0x47,
		0x82, 0xF2, 0x7F, 0x7A,
		0x26, 0x3A, 0x49, 0xC2,
		0x08,                               /* Number of controls in this terminal */
		0x01,                               /* Number of input pins in this terminal */
		0x02,                               /* Source ID : 2 : Connected to Proc Unit */
		0x03,                               /* Size of controls field for this terminal : 3 bytes */
#ifndef VIDEO_CONTROL_INTERFACE
		0x00, 0x00, 0x00,                   /* No controls supported */
#else
		0xFF, 0x00, 0x00,                   /* No controls supported */
#endif
		0x00,                               /* String descriptor index : Not used */

		/* Output Terminal Descriptor */
		0x09,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x03,                               /* Output Terminal Descriptor type */
		0x04,                               /* ID of this terminal */
		0x01, 0x01,                         /* USB Streaming terminal type */
		0x00,                               /* No association terminal */
		0x03,                               /* Source ID : 3 : Connected to Extn Unit */
		0x00,                               /* String desc index : Not used */

		/* Video Control Status Interrupt Endpoint Descriptor */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_CONTROL_STATUS_1,            /* Endpoint address and description: EP-2 IN */
		CY_U3P_USB_EP_INTR,                 /* Interrupt End point Type */
		0x40, 0x00,                         /* Max packet size: 64 bytes */
		0x01,                               /* Servicing interval */

		/* Super Speed Endpoint Companion Descriptor */
		0x06,                               /* Descriptor size */
		CY_U3P_SS_EP_COMPN_DESCR,           /* SS Endpoint Companion Descriptor Type */
		0x00,                               /* Max no. of packets in a Burst: 1 */
		0x00,                               /* Attribute: N.A. */
		0x40,                               /* Bytes per interval: 1024 */
		0x00,

		/* Class Specific Interrupt Endpoint Descriptor */
		0x05,                               /* Descriptor size */
		0x25,                               /* Class Specific Endpoint Descriptor Type */
		CY_U3P_USB_EP_INTR,                 /* End point Sub Type */
		0x40, 0x00,                         /* Max packet size = 64 bytes */

		/* Standard Video Streaming Interface Descriptor (Interface 1, Alternate Setting 0) */
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface Descriptor type */
		0x01,                               /* Interface number: 1 */
		0x00,                               /* Alternate setting number: 0 */
#ifdef USE_ISOCHRONOUS_ENDPOINT
		0x00,                               /* No endpoints. */
#else
		0x01,                               /* 1 Bulk endpoint */
#endif
		0x0E,                               /* Interface class : CC_VIDEO */
		0x02,                               /* Interface sub class : SC_VIDEOSTREAMING */
		0x00,                               /* Interface protocol code : PC_PROTOCOL_UNDEFINED */
		0x06,                               /* Interface descriptor string index */

		/* Class-specific Video Streaming Input Header Descriptor
		 * Increase one byte as and when you add a new format */
		0x0E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS I/f Type */
		0x01,                               /* Descriptor Subtype: Input Header */
		0x01,                               /* Format descriptor follows */
		SS_TOTAL_SIZE_CLASS_DSCR_L,         /* Total size of Class specific VS descr -Low Byte*/
		SS_TOTAL_SIZE_CLASS_DSCR_H,         /* Total size of Class specific VS descr -High Byte*/
		FX3_EP_BULK_VIDEO_1,                /* EP address for BULK video data: EP 3 IN */
		0x00,                               /* No dynamic format change supported */
		0x04,                               /* Output terminal ID : 4 */
		0x02,                               /* > Still image capture method supported */
		0x00,                               /* Hardware trigger NOT supported */
		0x00,                               /* Hardware to initiate still image capture not supported */
		0x01,                               /* Size of controls field : 1 byte */
		0x00,                               /* D2 : Compression quality supported - No compression*/

		/* Class specific Uncompressed VS format descriptor */
		0x1B,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS interface Type */
		0x04,                               /* Subtype : VS_FORMAT */
		SS_UYVY_FORMAT_INDEX,               /* Format desciptor index */
		SS_UYVY_TOTAL_NO_OF_RES,            /* Number of Frame Descriptors that follow this descriptor */
		/* GUID, globally unique identifier used to identify streaming-encoding format: UYVY  */
		0X55, 0X59, 0X56, 0X59,             /* MEDIASUBTYPE_UYVY GUID: 59555932-0000-0010-8000-00AA00389B71 */
		/*'Y' = 0x59, 'U' = 0x55, 'V' = 0x56,'2' = 0x32 */
		0X00, 0X00, 0X10, 0X00,
		0X80, 0X00, 0X00, 0XAA,
		0X00, 0X38, 0X9B, 0X71,
		0x10,                               /* Number of bits per pixel: 16*/
		SS_UYVY_OPTIMUM_FRAME_INDEX,        /* Optimum Frame Index for this stream: 3 1080p */
		0x00,                               /* X dimension of the picture aspect ratio; Non-interlaced */
		0x00,                               /* Y dimension of the pictuer aspect ratio: Non-interlaced */
		0x00,                               /* Interlace Flags: Progressive scanning, no interlace */
		0x00,                               /* duplication of the video stream restriction: 0 - no restriction */

		/* Class specific Uncompressed VS Frame Descriptor - UYVY_1920x1080 at 60 FPS*/
		0X1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type */
		0x05,                               /* Subtype:  frame interface */
		SS_UYVY_1920x1080_INDEX,            /* Frame Descriptor Index : SS_FC_UYVY_1920x1080_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x80, 0X07,                         /* Width in pixel: 1920 = 0x0780*/
		0x38, 0x04,                         /* Height in pixel: 1080= 0x0438 */
		0x00, 0x00, 0xA7, 0x76,             /* Min bit rate (bits/s): 1920 x 1080 x 16 (No Of Bits per Pixel)  x 60 (Min FrameRate) = 0x76A70000‬‬  = 1990656000*/
		0x00, 0x00, 0xA7, 0x76,             /* Max bit rate (bits/s): 1920 x 1080 x 16 (No Of Bits per Pixel)  x 60 (Max FrameRate) = 0x76A70000‬‬  = 1990656000*/
		0x00, 0x48, 0x3F, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 1920 x 1080 x 2 (No Of Bytes per Pixel) = 4147200 or 0x3F4800*/
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */

		/* Class specific Uncompressed VS Frame Descriptor - UYVY_1280x720 at 60 FPS*/
		0X1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type */
		0x05,                               /* Subtype:  frame interface */
		SS_UYVY_1280x720_INDEX,             /* Frame Descriptor Index : SS_FC_UYVY_1280x720_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x00, 0X05,                         /* Width in pixel: 1280 */
		0xD0, 0x02,                         /* Height in pixel: 720 */
		0x00, 0x00, 0xBC, 0x34,             /* Min bit rate (bits/s): 1280 x 720 x 16 (No Of Bits per Pixel)  x 60 (Min FrameRate) = 0x34BC0000 */
		0x00, 0x00, 0xBC, 0x34,             /* Max bit rate (bits/s): 1280 x 720 x 16 (No Of Bits per Pixel)  x 60 (Max FrameRate) = 0x34BC0000 */
		0x00, 0x20, 0x1C, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 1280 x 720 x 2 (No Of Bytes per Pixel) = 0x1C2000 */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */

		/* Class specific Uncompressed VS Frame Descriptor - UYVY_640x480 at 60 FPS*/
		0X1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type */
		0x05,                               /* Subtype:  frame interface */
		SS_UYVY_640x480_INDEX,              /* Frame Descriptor Index : SS_FC_UYVY_640x480_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x80, 0X02,                         /* Width in pixel: 640 = 0x0280 */
		0xE0, 0x01,                         /* Height in pixel: 480= 0x01E0 */
		0x00, 0x00, 0x94, 0x11,             /* Min bit rate (bits/s): 640 x 480 x 16 (No Of Bits per Pixel)  x 60 (Min FrameRate) = 0x11940000 */
		0x00, 0x00, 0x94, 0x11,             /* Max bit rate (bits/s): 640 x 480 x 16 (No Of Bits per Pixel)  x 60 (Max FrameRate) = 0x11940000 */
		0x00, 0x60, 0x09, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 640 x 480 x 2 (No Of Bytes per Pixel) = 0x96000 */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7  */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7  */

		/* Class specific Uncompressed VS Frame Descriptor - UYVY_640x640 at 60 FPS*/
		0X1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type */
		0x05,                               /* Subtype:  frame interface */
		SS_UYVY_640x640_INDEX,              /* Frame Descriptor Index : SS_FC_UYVY_640x640_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x80, 0X02,                         /* Width in pixel: 640 */
		0x80, 0x02,                         /* Height in pixel: 640 */
		0x00, 0x00, 0x70, 0x17,             /* Min bit rate (bits/s): 640 x 640 x 16 (No Of Bits per Pixel)  x 60 (Min FrameRate) = 0x17700000 */
		0x00, 0x00, 0x70, 0x17,             /* Max bit rate (bits/s): 640 x 640 x 16 (No Of Bits per Pixel)  x 60 (Max FrameRate) = 0x17700000 */
		0x00, 0x80, 0x0C, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 640 x 640 x 2 (No Of Bytes per Pixel) = 819200 = */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7 = 0x00028B0A */

		/* Class specific Uncompressed VS Frame Descriptor - UYVY_640x640 at 30 FPS*/
		0X1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type */
		0x05,                               /* Subtype:  frame interface */
		SS_UYVY_640x360_INDEX,              /* Frame Descriptor Index : SS_FC_UYVY_640x360_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x80, 0X02,                         /* Width in pixel: 640 */
		0x68, 0x01,                         /* Height in pixel: 360 */
		0x00, 0x00, 0x2F, 0x0D,             /* Min bit rate (bits/s): 640 x 360 x 16 (No Of Bits per Pixel)  x 60 (Min FrameRate) = 0x0D2F0000 */
		0x00, 0x00, 0x2F, 0x0D,             /* Max bit rate (bits/s): 640 x 360 x 16 (No Of Bits per Pixel)  x 60 (Max FrameRate) = 0x0D2F0000 */
		0x00, 0x08, 0x07, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 640 x 360 x 2 (No Of Byets per Pixel) = 460800 */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7  */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x0A, 0x8B, 0x02, 0x00,             /* Default frame interval (in 100ns units): (1/60)x10^7  */

		/* Still image descriptor -UYVY*/
		SS_UYVY_STILL_DESCR_SIZE,           /* Size of this descriptor*/
		0x24,                               /* CS_INTERFACE descriptor type*/
		0x03,                               /* VS_STILL_IMAGE_FRAME descriptor subtype*/
		0x00,                               /* If method 3 is used, endpoint address is mentioned here*/
		SS_UYVY_STILL_TOTAL_NO_OF_RES,      /* No. of frame Resolutions Follows*/
		0x80, 0X07,                         /* Width in pixel: 1920  = 0x0780 */
		0x38, 0x04,                         /* Height in pixel: 1080 = 0x0438 */
		0x00, 0X05,                         /* Width in pixel: 1280  = 0x0500*/
		0xD0, 0x02,                         /* Height in pixel: 720  = 0x02D0*/
		0x01,                               /* Compression patterns supported*/
		0x00,                               /* No compression*/

		/* Color matching descriptor - Required to remove USBview error */
		0x06,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS interface Type */
		0x0D,                               /* Subtype : VS_COLORFORMAT */
		0x01,                               /* bColorPrimaries */
		0x01,                               /* bTransferCharacteristics */
		0x04,                               /* MatrixCoefficients */


#ifdef USE_ISOCHRONOUS_ENDPOINT
		/* Standard Video Streaming Interface Descriptor (Interface 1, Alternate Setting 1) */
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface descriptor type */
		0x01,                               /* Interface number: 1 */
		0x01,                               /* Alternate setting number: 1 */
		0x01,                               /* 1 Isochronous endpoint */
		0x0E,                               /* Interface class : CC_VIDEO */
		0x02,                               /* Interface sub class : SC_VIDEOSTREAMING */
		0x00,                               /* Interface protocol code : PC_PROTOCOL_UNDEFINED */
		0x00,                               /* Interface descriptor string index */

		/* Endpoint descriptor for ISO streaming video data */
		0x07,                           /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
		FX3_EP_BULK_VIDEO_1,            /* Endpoint address and description */
		0x04 | CY_U3P_USB_EP_ISO,       /* ISO end point : Async */
		FX3_EP_BULK_VIDEO_PKT_SIZE_L,   /* FX3_EP_BULK_VIDEO_PKT_SIZE_L */
		FX3_EP_BULK_VIDEO_PKT_SIZE_H,   /* FX3_EP_BULK_VIDEO_PKT_SIZE_H */
		0x01,                           /* Servicing interval for data transfers */

		/* Super speed endpoint companion descriptor */
		0x06,                           /* Descriptor size */
		CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
		FX3_EP_ISO_VIDEO_SS_BURST,    /* Max no. of packets in a burst */
		FX3_EP_ISO_VIDEO_SS_MULT,     /* Mult setting: Number of bursts per service interval. */
		0x00,
		0x04 * (FX3_EP_ISO_VIDEO_SS_MULT + 1) * (FX3_EP_ISO_VIDEO_SS_BURST + 1),	/* Bytes per interval */

#else

		/* Endpoint Descriptor for BULK Streaming Video Data */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_BULK_VIDEO_1,                 /* Endpoint address and description: EP 3 IN */
		CY_U3P_USB_EP_BULK,                 /* BULK End point */
		FX3_EP_BULK_VIDEO_PKT_SIZE_L,       /* FX3_EP_BULK_VIDEO_PKT_SIZE_L */
		FX3_EP_BULK_VIDEO_PKT_SIZE_H,       /* FX3_EP_BULK_VIDEO_PKT_SIZE_H */
		0x00,                               /* Servicing interval for data transfers */

		/* Super Speed Endpoint Companion Descriptor */
		0x06,                               /* Descriptor size */
		CY_U3P_SS_EP_COMPN_DESCR,           /* SS Endpoint Companion Descriptor Type */
		0x0F,                               /* Max number of packets per burst: 12 */
		0x00,                               /* Attribute: Streams not defined */
		0x00,                               /* No meaning for bulk */
		0x00,
#endif


};

/* Standard High Speed Configuration Descriptor */
uint8_t CyFx3USBHSConfigDscr[] =
{
		/* Configuration Descriptor*/
		0x09,                               /* Descriptor Size */
		CY_U3P_USB_CONFIG_DESCR,            /* Configuration Descriptor Type */
		HS_TOTAL_SIZE_CONFIG_DSCR_L,        /* Length of this descriptor and all sub descriptors - low Byte */
		HS_TOTAL_SIZE_CONFIG_DSCR_H,        /* Length of this descriptor and all sub descriptors - high Byte */
		0x02,                               /* Number of interfaces */
		0x01,                               /* Configuration number */
		0x03,                               /* Configuration string index */
		0x80,                               /* Config characteristics - Bus Powered */
		0x0C,                               /* Max power consumption of device (in 8mA unit) : 96mA */

		/**************************************************************** interface 1***********************************************************/

		/* Interface Association Descriptor */
		0x08,                               /* Descriptor Size */
		FX3_INTRFC_ASSN_DESCR,              /* Interface Association Descriptor Type */
		0x00,                               /* Interface number of the VideoControl interface
		                                         that is associated with this function*/
		0x02,                               /* Number of contiguous Video interfaces that are
		                                         associated with this function */
		0x0E,                               /* Video Interface Class Code: CC_VIDEO */
		0x03,                               /* Subclass code: SC_VIDEO_INTERFACE_COLLECTION*/
		0x00,                               /* Protocol: PC_PROTOCOL_UNDEFINED*/
		0x06,                               /* String Descriptor index for interface */

		/* Standard Video Control Interface Descriptor (Interface 0, Alternate Setting 0)*/
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface Descriptor type */
		0x00,                               /* Index of this Interface */
		0x00,                               /* Alternate setting number */
		0x01,                               /* Number of end points - 1 Interrupt Endpoint*/
		0x0E,                               /* Video Interface Class Code: CC_VIDEO  */
		0x01,                               /* Interface sub class: SC_VIDEOCONTROL */
		0x00,                               /* Interface protocol code: PC_PROTOCOL_UNDEFINED.*/
		0x06,                               /* Interface descriptor string index */

		/* Class specific VC Interface Header Descriptor */
		0x0D,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class Specific Interface Descriptor type: CS_INTERFACE */
		0x01,                               /* Descriptor Sub type: VC_HEADER */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x01,                         /* Revision of UVC class spec: 1.0 - Legacy version */
		0x50, 0x00,                         /* Total Size of class specific descriptors
		                                         (till Output terminal) */
#else
		0x10, 0x01,                         /* Revision of UVC class spec: 1.1 - Minimum version required
		                                         for USB Compliance. Not supported on Windows XP*/
		0x51, 0x00,                         /* Total Size of class specific descriptors
		                                         (till Output terminal) */
#endif
		0x00, 0x6C, 0xDC, 0x02,             /* Clock frequency : 48MHz(Deprecated) */
		0x01,                               /* Number of streaming interfaces */
		0x01,                               /* VideoStreaming interface 1 belongs to this VideoControl interface*/

		/* Input (Camera) Terminal Descriptor */
		0x12,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* CS_INTERFACE */
		0x02,                               /* VC_INPUT_TERMINAL subtype */
		0x01,                               /* ID of this input terminal */
		0x01, 0x02,                         /* ITT_CAMERA type. This terminal is a camera
		                                         terminal representing the CCD sensor*/
		0x00,                               /* No association terminal */
		0x00,                               /* Unused */
		0x00, 0x00,                         /* No optical zoom supported */
		0x00, 0x00,                         /* No optical zoom supported */
		0x00, 0x00,                         /* No optical zoom supported */
		0x03,                               /* Size of controls field for this terminal : 3 bytes */
#ifndef VIDEO_CONTROL_INTERFACE
		0x00, 0x00, 0x00,                   /* No controls supported */
#else
		0x0A, 0x00, 0x00,
#endif

		/* Processing Unit Descriptor */
#ifndef FX3_UVC_1_0_SUPPORT
		0x0D,                               /* Descriptor size */
#else
		0x0C,                               /* Descriptor size */
#endif
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x05,                               /* Processing Unit Descriptor type: VC_PROCESSING_UNIT*/
		0x02,                               /* ID of this unit */
		0x01,                               /* Source ID: 1: Conencted to input terminal */
		0x00, 0x40,                         /* Digital multiplier */
		0x03,                               /* Size of controls field for this terminal: 3 bytes */
#ifndef VIDEO_CONTROL_INTERFACE
		0x00, 0x00, 0x00,                   /* No controls supported */
#else
		0x69, 0x10, 0x00,
#endif
		0x00,                               /* String desc index: Not used */
#ifndef FX3_UVC_1_0_SUPPORT
		0x00,                               /* Analog Video Standards Supported: None */
#endif

		/* Extension Unit Descriptor */
		0x1C,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x06,                               /* Extension Unit Descriptor type */
		0x03,                               /* ID of this terminal */
		0x4D, 0x9C, 0x7B, 0x2C,             /* 16 byte GUID */
		0xC2, 0xCC, 0xDA, 0x47,
		0x82, 0xF2, 0x7F, 0x7A,
		0x26, 0x3A, 0x49, 0xC2,
		0x00,                               /* Number of controls in this terminal */
		0x01,                               /* Number of input pins in this terminal */
		0x02,                               /* Source ID : 2 : Connected to Proc Unit */
		0x03,                               /* Size of controls field for this terminal : 3 bytes */
		0xFF, 0x00, 0x00,                   /* No controls supported */
		0x00,                               /* String descriptor index : Not used */

		/* Output Terminal Descriptor */
		0x09,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x03,                               /* Output Terminal Descriptor type */
		0x04,                               /* ID of this terminal */
		0x01, 0x01,                         /* USB Streaming terminal type */
		0x00,                               /* No association terminal */
		0x03,                               /* Source ID : 3 : Connected to Extn Unit */
		0x00,                               /* String desc index : Not used */

		/* Video Control Status Interrupt Endpoint Descriptor */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_CONTROL_STATUS_1,            /* Endpoint address and description: EP-2 IN */
		CY_U3P_USB_EP_INTR,                 /* Interrupt End point Type */
		0x40, 0x00,                         /* Max packet size: 64 bytes */
		0x01,                               /* Servicing interval */

		/* Class Specific Interrupt Endpoint Descriptor */
		0x05,                               /* Descriptor size */
		0x25,                               /* Class Specific Endpoint Descriptor Type */
		CY_U3P_USB_EP_INTR,                 /* End point Sub Type */
		0x40, 0x00,                         /* Max packet size = 64 bytes */

		/* Standard Video Streaming Interface Descriptor (Interface 1, Alternate Setting 0) */
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface Descriptor type */
		0x01,                               /* Interface number: 1 */
		0x00,                               /* Alternate setting number: 0 */
		0x01,                               /* Number of end points: 1 Bulk Endpoint */
		0x0E,                               /* Interface class : CC_VIDEO */
		0x02,                               /* Interface sub class : SC_VIDEOSTREAMING */
		0x00,                               /* Interface protocol code : PC_PROTOCOL_UNDEFINED */
		0x06,                               /* Interface descriptor string index */

		/* Class-specific Video Streaming Input Header Descriptor */
		0x0E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS I/f Type */
		0x01,                               /* Descriptor Subtype: Input Header */
		0x01,                               /* 2 format desciptor follows */
		HS_TOTAL_SIZE_CLASS_DSCR_L,         /* Total size of Class specific VS descr -Low Byte*/
		HS_TOTAL_SIZE_CLASS_DSCR_H,         /* Total size of Class specific VS descr -High Byte*/
		FX3_EP_BULK_VIDEO_1,                /* EP address for BULK video data: EP 3 IN */
		0x00,                               /* No dynamic format change supported */
		0x04,                               /* Output terminal ID : 4 */
		0x02,                               /* > Still image capture method supported */
		0x00,                               /* Hardware trigger NOT supported */
		0x00,                               /* Hardware to initiate still image capture not supported */
		0x01,                               /* Size of controls field : 1 byte */
		0x00,                               /* D2 : Compression quality supported - No compression*/

		/* Class specific Uncompressed VS format descriptor */
		0x1B,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS interface Type */
		0x04,                               /* Subtype : VS_FORMAT */
		0x01,                               /* Format desciptor index */
		HS_UYVY_TOTAL_NO_OF_RES,         /* Number of Frame Descriptors that follow this descriptor:  */
		/* GUID, globally unique identifier used to identify streaming-encoding format: UYVY  */
		0x55, 0x59, 0x56, 0x59,             /* MEDIASUBTYPE_UYVY GUID: 59555932-0000-0010-8000-00AA00389B71 */
		/* MEDIASUBTYPE_UYVY GUID: 55595659-0000-0010-8000-00AA00389B71 */
		/*'Y' = 0x59, 'U' = 0x55, 'V' = 0x56,'2' = 0x32 */
		0x00, 0x00, 0x10, 0x00,
		0x80, 0x00, 0x00, 0xAA,
		0x00, 0x38, 0x9B, 0x71,
		0x10,                               /* Number of bits per pixel: 16*/
		HS_UYVY_OPTIMUM_FRAME_INDEX,        /* Optimum Frame Index for this stream: 2 (720p) */
		0x00,                               /* X dimension of the picture aspect ratio; Non-interlaced */
		0x00,                               /* Y dimension of the pictuer aspect ratio: Non-interlaced */
		0x00,                               /* Interlace Flags: Progressive scanning, no interlace */
		0x00,                               /* duplication of the video stream restriction: 0 - no restriction */

		/* Class specific Uncompressed VS Frame Descriptor 1 - UYVY (640x480) at 30 FPS*/
		0x1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type*/
		0x05,                               /* Subtype:  frame interface*/
		HS_UYVY_640x480_INDEX,           /* Frame Descriptor Index: HS_UYVY_640x480_INDEX */
		0x00,                               /* No Still image capture method supported */
		0x80, 0x02,                         /* Width in pixel:  640 */
		0xE0, 0x01,                         /* Height in pixel: 480 */
		0x00, 0x00, 0xCA, 0x08,             /* Min bit rate (bits/s): 640 x 360 x No Of Bits per Pixel x FrameRate = 147456000 */
		0x00, 0x00, 0xCA, 0x08,             /* Max bit rate (bits/s): Fixed rate so same as Min */
		0x00, 0x60, 0x09, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 640 x 480 x 2 */
		0x15, 0x16, 0x05, 0x00,             /* Default frame interval (in 100ns units): (1/30)x10^7  */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x15, 0x16, 0x05, 0x00,             /* Frame interval 1 (in 100ns units): (1/30)x10^7  */

		/* Still image descriptor -UYVY*/
		HS_UYVY_STILL_DESCR_SIZE,           /* Size of this descriptor*/
		0x24,                               /* CS_INTERFACE descriptor type*/
		0x03,                               /* VS_STILL_IMAGE_FRAME descriptor subtype*/
		0x00,                               /* If method 3 is used, endpoint address is mentioned here*/
		0x01,                               /* No. of frame Resolutions Follows*/
		0x00, 0X05,                         /* Width in pixel: 1280 */
		0xD0, 0x02,                         /* Height in pixel: 720 */
		0x01,                               /* Compression patterns supported*/
		0x00,                               /* No compression*/

		/* Endpoint Descriptor for BULK Streaming Video Data */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_BULK_VIDEO_1,                /* Endpoint address and description: EP 3 IN */
		CY_U3P_USB_EP_BULK,                 /* BULK End point */
		0x00, 0x02,                         /* FX3_EP_BULK_VIDEO_PKT_SIZE_H */
		0x00,                               /* Servicing interval for data transfers */

};

/* Standard Full Speed Configuration Descriptor*/
/*The below Full Speed Configuration Descriptor is dummy descriptor.*/
const uint8_t CyFx3USBFSConfigDscr[] =
{
		/* Configuration descriptor */
		0x09,                               /* Descriptor size */
		CY_U3P_USB_CONFIG_DESCR,            /* Configuration descriptor type */
#ifdef FX3_UVC_1_0_SUPPORT
		0xEB, 0x00,                         /* Length of this descriptor and all sub descriptors */
#else
		0xEC, 0x00,                         /* Length of this descriptor and all sub descriptors */
#endif
		0x02,                               /* Number of interfaces */
		0x01,                               /* Configuration number */
		0x05,                               /* Configuration string index */
		0x80,                               /* Config characteristics - self powered */
		0x32,                               /* Max power consumption of device (in 2mA unit) : 100mA */

		/* Interface Association Descriptor */
		0x08,                               /* Descriptor Size */
		FX3_INTRFC_ASSN_DESCR,              /* Interface Association Descriptor Type */
		0x00,                               /* Interface number of the VideoControl interface
                                               that is associated with this function*/
		0x02,                               /* Number of contiguous Video interfaces that are
                                               associated with this function */
		0x0E,                               /* Video Interface Class Code: CC_VIDEO */
		0x03,                               /* Subclass code: SC_VIDEO_INTERFACE_COLLECTION*/
		0x00,                               /* Protocol: PC_PROTOCOL_UNDEFINED*/
		0x00,                               /* String Descriptor index for interface */

		/* Standard Video Control Interface Descriptor (Interface 0, Alternate Setting 0)*/
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR,            /* Interface Descriptor type */
		0x00,                               /* Index of this Interface */
		0x00,                               /* Alternate setting number */
		0x01,                               /* Number of end points - 1 Interrupt Endpoint*/
		0x0E,                               /* Video Interface Class Code: CC_VIDEO */
		0x01,                               /* Interface sub class: SC_VIDEOCONTROL */
		0x00,                               /* Interface protocol code: PC_PROTOCOL_UNDEFINED.*/
		0x00,                               /* Interface descriptor string index */

		/* Class specific VC Interface Header Descriptor */
		0x0D,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class Specific Interface Descriptor type: CS_INTERFACE */
		0x01,                               /* Descriptor Sub type: VC_HEADER */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x01,                         /* Revision of UVC class spec: 1.0 - Legacy version */
		0x50, 0x00,                         /* Total Size of class specific descriptors
                                         (till Output terminal) */
#else
		0x10, 0x01,                         /* Revision of UVC class spec: 1.1 - Minimum version required
                                               for USB Compliance. Not supported on Windows XP*/
		0x51, 0x00,                         /* Total Size of class specific descriptors
                                               (till Output terminal) */
#endif
		0x00, 0x6C, 0xDC, 0x02,             /* Clock frequency : 48MHz(Deprecated) */
		0x01,                               /* Number of streaming interfaces */
		0x01,                               /* VideoStreaming interface 1 belongs to this VideoControl interface*/

		/* Input (Camera) Terminal Descriptor */
		0x12,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* CS_INTERFACE */
		0x02,                               /* VC_INPUT_TERMINAL subtype */
		0x01,                               /* ID of this input terminal */
		0x01, 0x02,                         /* ITT_CAMERA type. This terminal is a camera
                                               terminal representing the CCD sensor*/
		0x00,                               /* No association terminal */
		0x00,                               /* Unused */
		0x00, 0x00,                         /* No optical zoom supported */
		0x00, 0x00,                         /* No optical zoom supported */
		0x00, 0x00,                         /* No optical zoom supported */
		0x03,                               /* Size of controls field for this terminal : 3 bytes */
		0x00, 0x00, 0x00,                   /* No controls supported */

		/* Processing Unit Descriptor */
#ifdef FX3_UVC_1_0_SUPPORT
		0x0C,                               /* Descriptor size */
#else
		0x0D,                               /* Descriptor size */
#endif
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x05,                               /* Processing Unit Descriptor type: VC_PROCESSING_UNIT*/
		0x02,                               /* ID of this unit */
		0x01,                               /* Source ID: 1: Conencted to input terminal */
		0x00, 0x40,                         /* Digital multiplier */
		0x03,                               /* Size of controls field for this terminal: 3 bytes */
		0x00, 0x00, 0x00,                   /* No controls supported */
		0x00,                               /* String desc index: Not used */
#ifndef FX3_UVC_1_0_SUPPORT
		0x00,                               /* Analog Video Standards Supported: None */
#endif


		/* Extension Unit Descriptor */
		0x1C,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x06,                               /* Extension Unit Descriptor type */
		0x03,                               /* ID of this terminal */
		0xFF, 0xFF, 0xFF, 0xFF,             /* 16 byte GUID */
		0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF,
		0x00,                               /* Number of controls in this terminal */
		0x01,                               /* Number of input pins in this terminal */
		0x02,                               /* Source ID : 2 : Connected to Proc Unit */
		0x03,                               /* Size of controls field for this terminal : 3 bytes */
		0x00, 0x00, 0x00,                   /* No controls supported */
		0x00,                               /* String descriptor index : Not used */

		/* Output Terminal Descriptor */
		0x09,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class specific interface desc type */
		0x03,                               /* Output Terminal Descriptor type */
		0x04,                               /* ID of this terminal */
		0x01, 0x01,                         /* USB Streaming terminal type */
		0x00,                               /* No association terminal */
		0x03,                               /* Source ID : 3 : Connected to Extn Unit */
		0x00,                               /* String desc index : Not used */

		/* Video Control Status Interrupt Endpoint Descriptor */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_CONTROL_STATUS_1,            /* Endpoint address and description: EP-2 IN */
		CY_U3P_USB_EP_INTR,                 /* Interrupt End point Type */
		0x40, 0x00,                         /* Max packet size: 64 bytes */
		0x01,                               /* Servicing interval */

		/* Class Specific Interrupt Endpoint Descriptor */
		0x05,                               /* Descriptor size */
		0x25,                               /* Class specific endpoint descriptor type */
		CY_U3P_USB_EP_INTR,                 /* End point sub type */
		0x40, 0x00,                         /* Max packet size = 64 bytes */

		/* Standard Video Streaming Interface Descriptor (Interface 1, Alternate Setting 0) */
		0x09,                               /* Descriptor size */
		CY_U3P_USB_INTRFC_DESCR, /* Interface descriptor type */
		0x01,                               /* Interface number: 1 */
		0x00,                               /* Alternate setting number: 0 */
		0x01,                               /* Number of end points: 1 Bulk endopoint*/
		0x0E,                               /* Interface class : CC_VIDEO */
		0x02,                               /* Interface sub class : SC_VIDEOSTREAMING */
		0x00,                               /* Interface protocol code : PC_PROTOCOL_UNDEFINED */
		0x00,                               /* Interface descriptor string index */

		/* Class-specific Video Streaming Input Header Descriptor */
		0x0E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Class-specific VS interface type */
		0x01,                               /* Descriptotor subtype : Input Header */
		0x01,                               /* 1 format desciptor follows */
		0x65, 0x00,                         /* Total size of class specific VS descr */
		FX3_EP_BULK_VIDEO_1,                /* EP address for BULK video data: EP 3 IN */
		0x00,                               /* No dynamic format change supported */
		0x04,                               /* Output terminal ID : 4 */
		0x00,                               /* No Still image capture method supported */
		0x00,                               /* Hardware trigger not supported */
		0x00,                               /* Hardware to initiate still image capture not supported*/
		0x01,                               /* Size of controls field : 1 byte */
		0x00,                               /* D2 : Compression quality supported - No compression */

		/* Class specific VS format descriptor */
		0x1B,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR, /* Class-specific VS interface Type */
		0x04,                               /* Subtype : VS_FORMAT */
		0x01,                               /* Format desciptor index */
		0x02,                               /* Number of Frame Descriptors that follow this descriptor: 2 */
		/* GUID, globally unique identifier used to identify streaming-encoding format: UYVY */
		0x59, 0x55, 0x59, 0x32,             /* MEDIASUBTYPE_UYVY GUID: 59555932-0000-0010-8000-00AA00389B71 */
		/* MEDIASUBTYPE_UYVY GUID: 55595659-0000-0010-8000-00AA00389B71 */
		/*'Y' = 0x59, 'U' = 0x55, 'V' = 0x56,'2' = 0x32 */
		0x00, 0x00, 0x10, 0x00,
		0x80, 0x00, 0x00, 0xAA,
		0x00, 0x38, 0x9B, 0x71,
		0x10,                               /* Number of bits per pixel: 16*/
		0x02,                               /* Optimum Frame Index for this stream: 1 (320x240) */
		0x00,                               /* X dimension of the picture aspect ratio; Non-interlaced */
		0x00,                               /* Y dimension of the pictuer aspect ratio: Non-interlaced */
		0x00,                               /* Interlace Flags: Progressive scanning, no interlace */
		0x00,                               /* duplication of the video stream restriction: 0 - no restriction */

		/* Class specific Uncompressed VS Frame Descriptor 1 - 320x240@5fps*/
		0x1E,                               /* Descriptor size */
		FX3_CS_INTRFC_DESCR,                /* Descriptor type*/
		0x05,                               /* Subtype: Uncompressed frame interface*/
		0x01,                               /* Frame Descriptor Index: 1 */
		0x00,                               /* No Still image capture method supported */
		0x40, 0x01,                         /* Width in pixel: 320 */
		0xF0, 0x00,                         /* Height in pixel: 240 */
		0x00, 0xC0, 0x5D, 0x00,             /* Min bit rate (bits/s): 320 x 240 x 2 x 5 x 8 = 6144000 */
		0x00, 0xC0, 0x5D, 0x00,             /* Max bit rate (bits/s): Fixed rate so same as Min */
		0x00, 0x58, 0x02, 0x00,             /* Maximum video or still frame size in bytes(Deprecated): 320 x 240 x 2*/
		0x80, 0x84, 0x1E, 0x00,             /* Default frame interval (in 100ns units): (1/5)x10^7 */
		0x01,                               /* Frame interval type : No of discrete intervals */
		0x80, 0x84, 0x1E, 0x00,             /* Frame interval 3: Same as Default frame interval */

		/* Endpoint descriptor for Bulk streaming video data */
		0x07,                               /* Descriptor size */
		CY_U3P_USB_ENDPNT_DESCR,            /* Endpoint Descriptor Type */
		FX3_EP_BULK_VIDEO_1,                /* Endpoint address and description: EP 3 IN */
		CY_U3P_USB_EP_BULK,                 /* BULK End point */
		0x40,                               /* EP Packet Size: 64 bytes */
		0x00,
		0x00                                /* Servicing interval for data transfers */
};

/* Standard language ID string descriptor */
const uint8_t CyFx3USBStringLangIDDscr[] =
{
		0x04,                               /* Descriptor size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		0x09, 0x04                          /* Language ID supported */
};


/* Standard manufacturer string descriptor */
const uint8_t CyFx3USBManufactureDscr[] =
{
		0x10,                               /* Descriptor size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		'C', 0x00,                          /* Manufacturer String */
		'y', 0x00,
		'p', 0x00,
		'r', 0x00,
		'e', 0x00,
		's', 0x00,
		's', 0x00
};


/* Standard product string descriptor */
const uint8_t CyFx3USBConfigSSDscr[] =
{
		0x10,                               /* Descriptor Size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		'U', 0x00,                          /* Super-Speed Configuration Descriptor */
		'S', 0x00,
		'B', 0x00,
		'-', 0x00,
		'3', 0x00,
		'.', 0x00,
		'0', 0x00
};

/* Standard product string descriptor */
const uint8_t CyFx3USBConfigHSDscr[] =
{
		0x10,                               /* Descriptor Size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		'U', 0x00,                          /* High-Speed Configuration Descriptor */
		'S', 0x00,
		'B', 0x00,
		'-', 0x00,
		'2', 0x00,
		'.', 0x00,
		'1', 0x00
};
/* Standard product string descriptor */
const uint8_t CyFx3USBConfigFSDscr[] =
{
		0x10,                               /* Descriptor Size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		'U', 0x00,                          /* Full-Speed Configuration Descriptor */
		'S', 0x00,
		'B', 0x00,
		'-', 0x00,
		'1', 0x00,
		'.', 0x00,
		'1', 0x00
};

/* Standard product string descriptor */
const uint8_t CyFx3USBProductDescr[] =
{
		0x1E,                               /* Descriptor Size */
		CY_U3P_USB_STRING_DESCR,            /* Device descriptor type */
		'F', 0x00,                          /* Product Descriptor String */
		'X', 0x00,
		'3', 0x00,
		' ', 0x00,
		'I', 0x00,
		'S', 0x00,
		' ', 0x00,
		'C', 0x00,
		'A', 0x00,
		'M', 0x00,
		' ', 0x00,
		'R', 0x00,
		'D', 0x00,
		'K', 0x00
};


/* Microsoft OS String Descriptor. */
const uint8_t CyFx3UsbMsOSStringDscr[] __attribute__ ((aligned (32))) =
{
		0x12,                       /* Length of the descriptor */
		CY_U3P_USB_STRING_DESCR,    /* String descriptor */
		'M', 0x00,                  /* MS OS Descriptor Signature */
		'S', 0x00,
		'F', 0x00,
		'T', 0x00,
		'1', 0x00,
		'0', 0x00,
		'0', 0x00,
		0x05,                      /* MS OS Descriptor Vendor Code */
		0x01
};

/* Microsoft OS Descriptor Custom Property Section to enable selective suspend Mode */
const uint8_t CyFx3UsbMsOSExtPropDscr[] __attribute__ ((aligned (32))) =
{
		0x4C, 0x00, 0x00, 0x00,     /* Length of the complete MS OS Descriptor: Header Section + Custom Property Section */
		0x00, 0x01,                 /* MS OS Descriptor Version */
		0x05, 0x00,                 /* Extended OS Descriptor type */
		0x01, 0x00,                 /* One custom Property */

		/* Custom property Section */
		0x42, 0x00, 0x00, 0x00,
		0x04, 0x00, 0x00, 0x00,
		0x30, 0x00,

		'S', 0x00,
		'e', 0x00,
		'l', 0x00,
		'e', 0x00,
		'c', 0x00,
		't', 0x00,
		'i', 0x00,
		'v', 0x00,
		'e', 0x00,

		'S', 0x00,
		'u', 0x00,
		's', 0x00,
		'p', 0x00,
		'e', 0x00,
		'n', 0x00,
		'd', 0x00,

		'E', 0x00,
		'n', 0x00,
		'a', 0x00,
		'b', 0x00,
		'l', 0x00,
		'e', 0x00,
		'd', 0x00,
		0x00, 0x00,

		0x04, 0x00, 0x00, 0x00,
		0x01, 0x00, 0x00, 0x00
};

/* UVC Probe Control Settings */
uint8_t glProbeCtrl_1920x1080_60fps[FX3_UVC_MAX_PROBE_SETTING] = {
		/* UVC Probe Control Setting - UYVY-1920x1080 - 60 FPS */
		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		SS_UYVY_1920x1080_INDEX,            /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x48, 0x3F, 0x00,             /* Maximum video or still frame size in bytes: 1920 x 1080 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};

/* UVC Probe Control Settings */
uint8_t glProbeCtrl_1280x720_60fps[FX3_UVC_MAX_PROBE_SETTING] = {

		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		SS_UYVY_1280x720_INDEX,             /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x20, 0x1C, 0x00,             /* Maximum video or still frame size in bytes: 1280 x 720 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};

/* UVC Probe Control Setting - UYVY-640x640 - 60 FPS */
uint8_t glProbeCtrl_640x640_60fps[FX3_UVC_MAX_PROBE_SETTING] = {

		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		SS_UYVY_640x640_INDEX,              /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x80, 0x0C, 0x00,             /* Maximum video or still frame size in bytes: 640 x 640 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};

/* UVC Probe Control Setting - UYVY-640x360 - 60 FPS */
uint8_t glProbeCtrl_640x360_60fps[FX3_UVC_MAX_PROBE_SETTING] = {

		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,	            /* Corresponding Video format index */
		SS_UYVY_640x360_INDEX,  	        /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x08, 0x07, 0x00,             /* Maximum video or still frame size in bytes: 640 x 360 x 2 = 0x00070800 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};


/* UVC Probe Control Setting - UYVY-640x480 - 60 FPS */
uint8_t glProbeCtrl_640x480_60fps[FX3_UVC_MAX_PROBE_SETTING] = {
		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		SS_UYVY_640x480_INDEX,              /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x60, 0x09, 0x00,             /* Maximum video or still frame size in bytes: 640 x 480 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};


/* UVC Probe Control Setting - UYVY-640x480 - 60 FPS */
uint8_t glProbeCtrl_640x480_60fps_HS[FX3_UVC_MAX_PROBE_SETTING] = {

		0x00, 0x00,                         /* bmHint : No fixed parameters */
		HS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		HS_UYVY_640x480_INDEX,              /* Corresponding Video frame index */
		0x0A, 0x8B, 0x02, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x60, 0x09, 0x00,             /* Maximum video or still frame size in bytes: 640 x 480 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};




/* Place holder for UVC Probe Control Settings */
uint8_t glProbeCtrl[FX3_UVC_MAX_PROBE_SETTING] = {
		/* UVC Probe Control Setting - UYVY-1920x1080 - 60 FPS */
		0x00, 0x00,                         /* bmHint : No fixed parameters */
		SS_UYVY_FORMAT_INDEX,               /* Corresponding Video format index */
		SS_UYVY_1920x1080_INDEX,            /* Corresponding Video frame index */
		0x15, 0x16, 0x05, 0x00,             /* Frame interval (in 100ns units): (1/60)x10^7  */
		0x00, 0x00,                         /* Key frame rate in key frame/video frame units */
		0x00, 0x00,                         /* PFrame rate in PFrame / key frame units */
		0x00, 0x00,                         /* Compression quality control */
		0x00, 0x00,                         /* Window size for average bit rate */
		0x00, 0x00,                         /* Internal video streaming i/f latency in ms */
		0x00, 0x48, 0x3F, 0x00,             /* Maximum video or still frame size in bytes: 1920 x 1080 x 2 */
#ifdef FX3_UVC_1_0_SUPPORT
		0x00, 0x90, 0x00, 0x00,             /* No. of bytes device can rx in single payload: 56KB */
#else
#ifdef USE_ISOCHRONOUS_ENDPOINT
		FX3_UVC_ISO_PAYLOAD_SIZE_L, FX3_UVC_ISO_PAYLOAD_SIZE_H, 0x00, 0x00,/* No. of bytes device can rx in single payload */
#else
		0x00, 0xE0, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
		0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
		0x00,                               /* Framing Information - Ignored for uncompressed format */
		0x00,                               /* Preferred payload format version */
		0x00,                               /* Minimum payload format version */
		0x00,                               /* Maximum payload format version */
#ifdef FX3_UVC_1_5_SUPPORT
		0x00,                               /* Current bUsage */
		0x00,                               /* bBitDepthLuma */
		0x00,                               /* bmSettings */
		0x00,                               /* bMaxNumberOfRefFramesPlus1 */
		0x00, 0x00,                         /* bmRateControlModes*/
		0x00, 0x00, 0x00, 0x00,             /* bmLayoutPerStream */
		0x00, 0x00, 0x00, 0x00,             /* No. of bytes device can rx in single payload */
#endif
#endif
};


/* [ ] */
