/*
 ## Cypress FX3 Firmware Example Source (cyFX3_videocontrol.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2013-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *
 *  Created on: 30-June-2015
 *  Author: savj
 */

/******************************Video Control Interface************************/
#include <cyu3error.h>
#include <cyu3i2c.h>
#include "cyu3_isp.h"
#include <cyu3utils.h>
#include <cyu3gpio.h>
#include <cyu3usb.h>
#include "cyu3spi.h"
#include "cyfx3_uvc.h"
#include "cyfx3_videocontrol.h"
#include "cyfx3_videostreaming.h"

#include "stddef.h"
#include "string.h"

#ifdef VIDEO_CONTROL_INTERFACE

#ifdef ISP_VIDEO_CONTROL_DEBUG
#define UvcAppDebugPrint CyU3PDebugPrint

#else
#define UvcAppDebugPrint(...)

#endif

/*Video Control Features- Camera Terminal- Struct ure Definitions*/
VideoControl_uint32_t autoExpCtrl      = {0x2,0x2,0x4,0x6,0x2,0x2};
VideoControl_uint32_t manExpCtrl       = {0x00001000, 0x00001000, 0x0000FFFF, 0x00000005, 0x00001000, 0x00001000};


/*Video Control Features- Processing Unit- Structure Definitions*/
/*All controls are kept of 4 bytes as the ISP supports 4 byte values. WrBuff is used based on the UVC control length specified*/
static VideoControl_uint32_t mwbTempCtrl      = {0x00001000, 0x00000000, 0x00002000, 0x00000100, 0x00001000, 0x00001000};
static VideoControl_uint32_t awbTempCtrl      = {0x00000001, 0x00000000, 0x00000001, 0x00000001, 0x00000001, 0x00000001};
static VideoControl_uint32_t brightnessCtrl   = {0x000000C0, 0x00000064, 0x00000200, 0x00000001, 0x000000C0, 0x000000C0};
static VideoControl_uint32_t contrastCtrl     = {0x00000060, 0x00000001, 0x00000170, 0x00000001, 0x00000060, 0x00000060};
static VideoControl_uint32_t gammaCtrl        = {0x0000003C, 0x00000001, 0x000000A4, 0x00000001, 0x0000003C, 0x0000003C};
static VideoControl_uint32_t saturationCtrl   = {0x00000082, 0x00000064, 0x00000170, 0x00000001, 0x00000082, 0x00000082};

/*Video Control Features- Extension Unit- Structure Definitions*/
VideoControl_xuSensIspRegRdWrData_t xuI2CWriteCtrl, xuI2CReadCtrl;

/*Variable to keep a track of the error codes in video control requests*/
static	uint8_t glerrorstatus = VC_ERROR_CODE_NO_ERROR;

/* Variable to select the front or rear camera interrupt endpoint.
 * CyFalse: Means the front camera is selected.
 * CyTrue: Means the Rear camera sends out the INT packet payload*/
CyBool_t isMsXuVcRqstForRC = CyFalse;

static uint8_t VC_Get_Info = 0x03;

CyU3PReturnStatus_t HandleErrorCodeReq(uint8_t bRequest, uint8_t interfaceType)
{
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		CyU3PDebugPrint(4,"\n\rglerrorstatus = 0x%x", glerrorstatus);
		status = CyU3PUsbSendEP0Data(1,&glerrorstatus);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
	}
	break;
	}

	return status;
}

/* Requests to be supported for all the Video Control features
FX3_USB_UVC_GET_CUR_REQ
FX3_USB_UVC_GET_MIN_REQ
FX3_USB_UVC_GET_MAX_REQ
FX3_USB_UVC_GET_RES_REQ
FX3_USB_UVC_GET_DEF_REQ
FX3_USB_UVC_GET_LEN_REQ
FX3_USB_UVC_SET_CUR_REQ
 */
/*Function to handle Extension unit UVC Requests for I2C writes to sensor/ISP */
CyU3PReturnStatus_t HandleXUI2CWriteControlReq(uint8_t bRequest)
{
	uint16_t readCount = 0;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;
	uint16_t len = sizeof(xuSensIspRegRdWrData_t)/sizeof(uint8_t);
	uint8_t *WrBuff;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CWriteCtrl.CurrentVal;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Cur:");
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.opStatus = 0x%x ", xuI2CWriteCtrl.CurrentVal.opStatus);

		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.sensIspAddr = 0x%x ", xuI2CWriteCtrl.CurrentVal.sensIspAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.regAddr = 0x%x ", xuI2CWriteCtrl.CurrentVal.regAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.regData = 0x%x ", xuI2CWriteCtrl.CurrentVal.regData);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.regLen = 0x%x ", xuI2CWriteCtrl.CurrentVal.regLen);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.CurrentVal.dataLen = 0x%x ", xuI2CWriteCtrl.CurrentVal.dataLen);
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CWriteCtrl.MinVal;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Min");
		CyU3PMemSet(WrBuff, 0x0, len);
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CWriteCtrl.MaxVal;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Max");
		CyU3PMemSet(WrBuff, 0xFF, len);
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CWriteCtrl.Res;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Resolution");
		CyU3PMemSet(WrBuff, 0x01, len);
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		/* Both GET and SET requests are supported, auto modes not supported
		 *  D0  1=Supports GET value requests  Capability	SUPPORT GET REQUEST
		 *  D1  1=Supports SET value requests  Capability	SUPPORT SET REQUEST
		 *  D2  1=Disabled due to automatic mode (under device control) State	NOT SUPPORT
		 *  D3  1= Autoupdate Control	NOT SUPPORT
		 *  D4  1= Asynchronous Control	NOT SUPPORT
		 *  D7..D5  Reserved (Set to 0)	RESERVE AS ZEROS  */
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Info : 0x%x", VC_Get_Info);

		status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_BYTE, &VC_Get_Info);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CWriteCtrl.DefaultVal;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Default");
		CyU3PMemSet(WrBuff, 0x1, len);
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ :
	{
		/* Return the length */
		WrBuff = (uint8_t *)&len;
		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Get Len : %x ", len);

		status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD, WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		status = CyU3PUsbGetEP0Data(32, (uint8_t *)&xuI2CWriteCtrl.SetVal, &readCount);    //Always read more data from this API
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Get EP0 Data failed = 0x%X", status);
			return status;
		}

		UvcAppDebugPrint(4,"\n\rXU I2C WRITE Set Cur :");
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.SetVal.sensIspAddr = 0x%x ", xuI2CWriteCtrl.SetVal.sensIspAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.SetVal.regAddr = 0x%x ", xuI2CWriteCtrl.SetVal.regAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.SetVal.regData = 0x%x ", xuI2CWriteCtrl.SetVal.regData);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.SetVal.regLen = 0x%x ", xuI2CWriteCtrl.SetVal.regLen);
		UvcAppDebugPrint(4,"\n\rxuI2CWriteCtrl.SetVal.dataLen = 0x%x ", xuI2CWriteCtrl.SetVal.dataLen);

		CyU3PMemSet((uint8_t*)&xuI2CWriteCtrl.CurrentVal, 0x0, len);

		xuI2CWriteCtrl.CurrentVal.opStatus = FX3_UVC_XU_DEV_PRO_PREV_SET_CUR;
		xuI2CWriteCtrl.CurrentVal.sensIspAddr = xuI2CWriteCtrl.SetVal.sensIspAddr;
		xuI2CWriteCtrl.CurrentVal.regAddr = xuI2CWriteCtrl.SetVal.regAddr;
		xuI2CWriteCtrl.CurrentVal.regData = xuI2CWriteCtrl.SetVal.regData;
		xuI2CWriteCtrl.CurrentVal.regLen =  xuI2CWriteCtrl.SetVal.regLen;
		xuI2CWriteCtrl.CurrentVal.dataLen = xuI2CWriteCtrl.SetVal.dataLen;

		status = CyFx3_Sensor_Isp_Reg_I2c_Write(&xuI2CWriteCtrl.SetVal);
		xuI2CWriteCtrl.CurrentVal.opStatus = status;
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	status = CyU3PUsbSendEP0Data(len, WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/* Requests to be supported for all the Video Control features
FX3_USB_UVC_GET_CUR_REQ
FX3_USB_UVC_GET_MIN_REQ
FX3_USB_UVC_GET_MAX_REQ
FX3_USB_UVC_GET_RES_REQ
FX3_USB_UVC_GET_DEF_REQ
FX3_USB_UVC_GET_LEN_REQ
FX3_USB_UVC_SET_CUR_REQ
 */
/*Function to handle Extension unit UVC Requests for I2C writes to sensor/ISP */
CyU3PReturnStatus_t HandleXUI2CReadControlReq(uint8_t bRequest)
{
	uint16_t readCount = 0;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;
	uint16_t len = sizeof(xuSensIspRegRdWrData_t)/sizeof(uint8_t);
	uint8_t *WrBuff;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CReadCtrl.CurrentVal;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Cur:");
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.opStatus = 0x%x ", xuI2CReadCtrl.CurrentVal.opStatus);

		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.sensIspAddr = 0x%x ", xuI2CReadCtrl.CurrentVal.sensIspAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.regAddr = 0x%x ", xuI2CReadCtrl.CurrentVal.regAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.regData = 0x%x ", xuI2CReadCtrl.CurrentVal.regData);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.regLen = 0x%x ", xuI2CReadCtrl.CurrentVal.regLen);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.CurrentVal.dataLen = 0x%x ", xuI2CReadCtrl.CurrentVal.dataLen);
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CReadCtrl.MinVal;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Min");
		CyU3PMemSet(WrBuff, 0x0, len);
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CReadCtrl.MaxVal;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Max");
		CyU3PMemSet(WrBuff, 0xFF, len);
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CReadCtrl.Res;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Resolution");
		CyU3PMemSet(WrBuff, 0x01, len);
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Info : 0x%x", VC_Get_Info);

		status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_BYTE, &VC_Get_Info);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		WrBuff = (uint8_t *)&xuI2CReadCtrl.DefaultVal;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Default");
		CyU3PMemSet(WrBuff, 0x1, len);
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ :
	{
		/* Return the length */
		WrBuff = (uint8_t *)&len;
		UvcAppDebugPrint(4,"\n\rXU I2C READ Get Len : %x ", len);

		status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD, WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		status = CyU3PUsbGetEP0Data(32, (uint8_t *)&xuI2CReadCtrl.SetVal, &readCount);    //Always read more data from this API
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Get EP0 Data failed = 0x%X", status);
			return status;
		}

		UvcAppDebugPrint(4,"\n\rXU I2C READ Set Cur :");
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.SetVal.sensIspAddr = 0x%x ", xuI2CReadCtrl.SetVal.sensIspAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.SetVal.regAddr = 0x%x ", xuI2CReadCtrl.SetVal.regAddr);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.SetVal.regData = 0x%x ", xuI2CReadCtrl.SetVal.regData);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.SetVal.regLen = 0x%x ", xuI2CReadCtrl.SetVal.regLen);
		UvcAppDebugPrint(4,"\n\rxuI2CReadCtrl.SetVal.dataLen = 0x%x ", xuI2CReadCtrl.SetVal.dataLen);

		CyU3PMemSet((uint8_t*)&xuI2CReadCtrl.CurrentVal, 0x0, len);

		xuI2CReadCtrl.CurrentVal.opStatus = FX3_UVC_XU_DEV_PRO_PREV_SET_CUR;
		xuI2CReadCtrl.CurrentVal.sensIspAddr = xuI2CReadCtrl.SetVal.sensIspAddr;
		xuI2CReadCtrl.CurrentVal.regAddr = xuI2CReadCtrl.SetVal.regAddr;
		xuI2CReadCtrl.CurrentVal.regLen =  xuI2CReadCtrl.SetVal.regLen;
		xuI2CReadCtrl.CurrentVal.dataLen = xuI2CReadCtrl.SetVal.dataLen;

		status = CyFx3_Sensor_Isp_Reg_I2c_Read(&xuI2CReadCtrl.CurrentVal);
		xuI2CReadCtrl.CurrentVal.opStatus = status;
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	status = CyU3PUsbSendEP0Data(len, WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/* Requests to be supported for all the Video Control features
FX3_USB_UVC_GET_CUR_REQ
FX3_USB_UVC_GET_MIN_REQ
FX3_USB_UVC_GET_MAX_REQ
FX3_USB_UVC_GET.Res_REQ
FX3_USB_UVC_GET_DEF_REQ
FX3_USB_UVC_SET_CUR_REQ
 */
/*Function to handle UVC Requests for Auto-Exposure Mode Selection*/
CyU3PReturnStatus_t HandleAEControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint8_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
				WrBuff = (uint8_t)autoExpCtrl.CurrentVal;
		}
		else
		{

			if(status != CY_U3P_SUCCESS)
				return status;
		}
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Min  = " );
		WrBuff = (uint8_t)autoExpCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Max  = " );
		WrBuff = (uint8_t)autoExpCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Res  = ");
		WrBuff = (uint8_t)autoExpCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_BYTE;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\rAE Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);

		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAE Get Def  = ");
		WrBuff = (uint8_t)autoExpCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&autoExpCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				if ((autoExpCtrl.SetVal == 0x00 ) || /*(aeModeFrontCam.SetVal == 0x01 ) ||*/ (autoExpCtrl.SetVal == 0x03) || (autoExpCtrl.SetVal == 0x05 )
						|| (autoExpCtrl.SetVal == 0x06 ) || (autoExpCtrl.SetVal == 0x07 ) || (autoExpCtrl.SetVal == 0x10 ))
				{
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
				}
				else if ((autoExpCtrl.SetVal == FX3_UVC_VC_AE_AUTO_MODE_VALUE ) /* Auto mode */
						|| (autoExpCtrl.SetVal == FX3_UVC_VC_AE_MANUAL_MODE_VALUE )) /* Shutter Priority-Manual mode */
				{
					CyU3PDebugPrint(4,"\r\n AE set cur val = %d", autoExpCtrl.SetVal);
					status = CyFx3_Isp_AEModeControl(&autoExpCtrl.SetVal, CyTrue);
					autoExpCtrl.CurrentVal = autoExpCtrl.SetVal;


					glerrorstatus = VC_ERROR_CODE_NO_ERROR;
				}
			}
		}
	}
	return status;
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_BYTE,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/*Function to handle UVC Requests for  Manual Exposure Control*/
CyU3PReturnStatus_t HandleExpAbsControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint32_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Cur = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			WrBuff = (uint32_t)manExpCtrl.CurrentVal;
		}
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Min  = " );
		WrBuff = (uint32_t)manExpCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Max  = " );
		WrBuff = (uint32_t)manExpCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Res  = ");
		WrBuff = (uint32_t)manExpCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_DWORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			if (manExpCtrl.CurrentVal == FX3_UVC_VC_AE_AUTO_MODE_VALUE)
				VC_Get_Info=0x0F;
			else
				VC_Get_Info=0x03;
		}

		UvcAppDebugPrint(4,"\n\rExpAbs Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);
		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\rExpAbs Get Def  = ");
		WrBuff = (uint32_t)manExpCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			if (autoExpCtrl.CurrentVal == FX3_UVC_VC_AE_AUTO_MODE_VALUE)
			{
				glerrorstatus = VC_ERROR_CODE_WRONG_STATE;
				return status;
			}

			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&manExpCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				if ((manExpCtrl.SetVal >= manExpCtrl.MinVal) && (manExpCtrl.SetVal <= manExpCtrl.MaxVal))
				{
					UvcAppDebugPrint(4,"\n\rExpAbs Set Cur  = %d", manExpCtrl.SetVal);
					status = CyFx3_Isp_ManualExposureControl(&manExpCtrl.SetVal, FX3_UVC_VC_REG_WRITE);
					if(status != CY_U3P_SUCCESS)
						return status;
					else
						manExpCtrl.CurrentVal = manExpCtrl.SetVal;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);
	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_DWORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}


/*Function to handle UVC Requests for Brightness control*/
CyU3PReturnStatus_t HandleBrightnessControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint16_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightness Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		status = CyFx3_Isp_BrightnessControl(&brightnessCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
		if(status != CY_U3P_SUCCESS)
			return status;
		else
			WrBuff = (uint16_t)brightnessCtrl.CurrentVal;

	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightness Get Min  = " );
		WrBuff = (uint16_t)brightnessCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightness Get Max  = " );
		WrBuff = (uint16_t)brightnessCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightness Get Res  = ");
		WrBuff = (uint16_t)brightnessCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightness Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_WORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\rBrightness Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);

		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\rBrightnessB Get Def  = ");
		WrBuff = (uint16_t)brightnessCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&brightnessCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\rBrightness Set Cur  = %d", brightnessCtrl.SetVal);
				if ((brightnessCtrl.SetVal >= brightnessCtrl.MinVal) && (brightnessCtrl.SetVal <= brightnessCtrl.MaxVal))
				{
					status = CyFx3_Isp_BrightnessControl(&brightnessCtrl.SetVal, FX3_UVC_VC_REG_WRITE); /*Change the Brightness Control*/
					brightnessCtrl.CurrentVal = brightnessCtrl.SetVal;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}


/*Function to handle UVC Requests for auto white balance control*/
CyU3PReturnStatus_t HandleAWBTempControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint8_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyFx3_Isp_AwbModeControl(&awbTempCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
			if(status != CY_U3P_SUCCESS)
				return status;
			else
			{
				if(awbTempCtrl.CurrentVal == 0x05)
				{
					WrBuff = 0x0;
				}
				else
				{
					WrBuff = 0x1;
				}
			}
		}
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Min  = " );
		/* As min max value of Front and Rear cam is same we send the FFC Min, Max value */
		WrBuff = (uint8_t)awbTempCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Max  = " );
		/* As min max value of Front and Rear cam is same we send the FFC Min, Max value */
		WrBuff = (uint8_t)awbTempCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Res Stalled");
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_BYTE;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		/* As both cam supports, get and set value we send 0x03*/
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\rAWB Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);

		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\rAWB Get Def  = ");

		/* As Default value of Front and Rear cam is same we send the FFC Default value */
		WrBuff = (uint8_t)awbTempCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&awbTempCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Get EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\rAWB Set Cur  = 0x%X", awbTempCtrl.SetVal);
				if ((awbTempCtrl.SetVal == FX3_UVC_VC_AWB_MANUAL_MODE_VALUE) || (awbTempCtrl.SetVal == FX3_UVC_VC_AWB_AUTO_MODE_VALUE))
				{
					awbTempCtrl.CurrentVal = awbTempCtrl.SetVal;
					status = CyFx3_Isp_AwbModeControl(&awbTempCtrl.SetVal, FX3_UVC_VC_REG_WRITE);
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_BYTE,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;

}

CyU3PReturnStatus_t HandleMWBTempControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint16_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Cur  = ");
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyFx3_Isp_MWBTempValue(&mwbTempCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
			if(status != CY_U3P_SUCCESS)
				return status;
			else
				WrBuff = (uint16_t)mwbTempCtrl.CurrentVal;
		}
	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Min  = " );
		WrBuff = (uint16_t)mwbTempCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Max  = " );
		WrBuff = (uint16_t)mwbTempCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Res  = ");
		WrBuff = (uint16_t)mwbTempCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Len = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_WORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;

		UvcAppDebugPrint(4,"\n\rMWB Temp Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);
		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}

		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\rMWB Temp Get Def  = ");
		WrBuff = (uint16_t)mwbTempCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			if (mwbTempCtrl.CurrentVal == FX3_UVC_VC_AWB_AUTO_MODE_VALUE)
			{
				glerrorstatus = VC_ERROR_CODE_WRONG_STATE;
				return status;
			}

			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&mwbTempCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Get EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\rMWB Temp Set Cur  = 0x%X", mwbTempCtrl.SetVal);
				if ((mwbTempCtrl.SetVal >= mwbTempCtrl.MinVal) && (mwbTempCtrl.SetVal <= mwbTempCtrl.MaxVal))
				{

					if(mwbTempCtrl.SetVal % mwbTempCtrl.Res == 0)
					{
						status = CyFx3_Isp_MWBTempValue(&mwbTempCtrl.SetVal, FX3_UVC_VC_REG_WRITE);
					}
					else
						glerrorstatus = VC_ERROR_CODE_UNKNOWN;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;

}


/*Function to handle UVC Requests for contrast control*/
CyU3PReturnStatus_t HandleContrastControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint16_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyFx3_Isp_ContrastControl(&contrastCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
			if(status != CY_U3P_SUCCESS)
				return status;
			else
				WrBuff = (uint16_t)contrastCtrl.CurrentVal;
		}

	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Min  = " );
		WrBuff = (uint16_t)contrastCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Max  = " );
		WrBuff = (uint16_t)contrastCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Res  = ");
		WrBuff = (uint16_t)contrastCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_WORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\r Contrast Get Info = %d", VC_Get_Info);
		WrBuff = VC_Get_Info;
		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Contrast Get Def  = ");
		WrBuff = (uint16_t)contrastCtrl.DefaultVal;

	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&contrastCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\rContrast Set Cur  = %d", contrastCtrl.SetVal);
				if ((contrastCtrl.SetVal >= contrastCtrl.MinVal) && (contrastCtrl.SetVal <= contrastCtrl.MaxVal))
				{
					status = CyFx3_Isp_ContrastControl(&contrastCtrl.SetVal, FX3_UVC_VC_REG_WRITE); /*Change the Contrast Control*/
					contrastCtrl.CurrentVal = contrastCtrl.SetVal;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;

			}
		}

		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/*Function to handle UVC Requests for gamma control*/
CyU3PReturnStatus_t HandleGammaControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint16_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		status = CyFx3_Isp_GammaControl(&gammaCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
		if(status != CY_U3P_SUCCESS)
			return status;
		else
			WrBuff = (uint16_t)gammaCtrl.CurrentVal;

	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma Get Min  = " );
		WrBuff = (uint16_t)gammaCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma Get Max  = " );
		WrBuff = (uint16_t)gammaCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma Get Res  = ");
		WrBuff = (uint16_t)gammaCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_WORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\r gamma Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);

		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\r gamma bal Get Def  = ");
		WrBuff = (uint16_t)gammaCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&gammaCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\r gamma Set Cur  = %d", gammaCtrl.SetVal);
				if ((gammaCtrl.SetVal >= gammaCtrl.MinVal) && (gammaCtrl.SetVal <= gammaCtrl.MaxVal))
				{
					status = CyFx3_Isp_GammaControl(&gammaCtrl.SetVal, FX3_UVC_VC_REG_WRITE); /*Change the Brightness Control*/
					gammaCtrl.CurrentVal = gammaCtrl.SetVal;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/*Function to handle UVC Requests for saturation control*/
CyU3PReturnStatus_t HandleSaturationControlReq(uint8_t bRequest, uint8_t interfaceType)
{
	uint16_t readCount = 0;
	uint32_t WrBuff;
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	switch (bRequest)
	{
	case FX3_USB_UVC_GET_CUR_REQ:
	{
		UvcAppDebugPrint(4,"\n\r saturation Get Cur  = " );
		if (glerrorstatus == VC_ERROR_CODE_OUT_OF_RANGE)
			return status;

		status = CyFx3_Isp_SaturationControl((uint16_t*)&saturationCtrl.CurrentVal, FX3_UVC_VC_REG_READ);
		if(status != CY_U3P_SUCCESS)
			return status;
		else
			WrBuff = (uint16_t)saturationCtrl.CurrentVal;

	}
	break;

	case FX3_USB_UVC_GET_MIN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r saturation Get Min  = " );
		WrBuff = (uint16_t)saturationCtrl.MinVal;
	}
	break;

	case FX3_USB_UVC_GET_MAX_REQ:
	{
		UvcAppDebugPrint(4,"\n\r saturation Get Max  = " );
		WrBuff = (uint16_t)saturationCtrl.MaxVal;
	}
	break;

	case FX3_USB_UVC_GET_RES_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Saturation Get Res  = ");
		WrBuff = (uint16_t)saturationCtrl.Res;
	}
	break;

	case FX3_USB_UVC_GET_LEN_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Saturation Get Len  = ");
		WrBuff = FX3_UVC_VC_LEN_VALUE_WORD;
	}
	break;

	case FX3_USB_UVC_GET_INFO_REQ:
	{
		VC_Get_Info=0x03;
		UvcAppDebugPrint(4,"\n\r Saturation Get Info  = ");
		WrBuff = VC_Get_Info;
		UvcAppDebugPrint(4," %d", WrBuff);

		status = CyU3PUsbSendEP0Data(1,(uint8_t*)&WrBuff);
		if (status != CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
		}
		return status;
	}
	break;

	case FX3_USB_UVC_GET_DEF_REQ:
	{
		UvcAppDebugPrint(4,"\n\r Saturation Get Def  = ");
		WrBuff = (uint16_t)saturationCtrl.DefaultVal;
	}
	break;

	case FX3_USB_UVC_SET_CUR_REQ:
	{
		if(interfaceType == FX3_UVC_CONTROL_INTERFACE_1)
		{
			status = CyU3PUsbGetEP0Data(16,(uint8_t*)&saturationCtrl.SetVal,&readCount);
			if (status != CY_U3P_SUCCESS)
			{
				CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
			}
			else
			{
				UvcAppDebugPrint(4,"\n\r saturation Set Cur  = %d", saturationCtrl.SetVal);
				if ((saturationCtrl.SetVal >= saturationCtrl.MinVal) && (saturationCtrl.SetVal <= saturationCtrl.MaxVal))
				{
					status = CyFx3_Isp_SaturationControl((uint16_t*)&saturationCtrl.SetVal, FX3_UVC_VC_REG_WRITE); /*Change the Brightness Control*/
					saturationCtrl.CurrentVal = saturationCtrl.SetVal;
				}
				else
					glerrorstatus = VC_ERROR_CODE_OUT_OF_RANGE;
			}
		}
		return status;
	}
	break;

	default:
		glerrorstatus = VC_ERROR_CODE_INVALID_REQUEST;
		return status;
		break;
	}

	UvcAppDebugPrint(4," %d", WrBuff);

	status = CyU3PUsbSendEP0Data(FX3_UVC_VC_LEN_VALUE_WORD,(uint8_t*)&WrBuff);
	if (status != CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4,"\n\r Send EP0 Data failed = 0x%X", status);
	}

	return status;
}

/*Function to handle all supported UVC Video Control requests*/
CyU3PReturnStatus_t CyFx3UvcHandleVCInterfaceReq(uint16_t wIndex, uint16_t wValue, uint16_t bRequest)
{
	CyU3PReturnStatus_t status = CY_U3P_ERROR_BAD_ARGUMENT;

	if (CY_U3P_GET_MSB(wIndex) == FX3_UVC_VC_CAMERA_TERMINAL_ID) /*Video Control- Camera Teminal*/
	{
		UvcAppDebugPrint(4,"\n\r\n\rVC:T ID 0x%x CS 0x%x Req 0x%x ", CY_U3P_GET_MSB(wIndex), wValue, bRequest);

		switch (CY_U3P_GET_MSB(wValue))
		{
		case CT_AE_MODE_CONTROL: /*Auto-Exposure Mode Selection*/
			status = HandleAEControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case CT_EXPOSURE_TIME_ABSOLUTE_CONTROL: /*Manual Exposure Control*/
			status = HandleExpAbsControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		default:
			glerrorstatus = VC_ERROR_CODE_INVALID_CONTROL;
			break;
		}
	}
	else if (CY_U3P_GET_MSB(wIndex) == FX3_UVC_VC_PROCESSING_UNIT_ID) /*Video Control- Processing Unit*/
	{

		UvcAppDebugPrint(4,"\n\r\n\rVC:T ID 0x%x CS 0x%x Req 0x%x ", CY_U3P_GET_MSB(wIndex), wValue, bRequest);

		switch (CY_U3P_GET_MSB(wValue))
		{
		case PU_BRIGHTNESS_CONTROL: /*Brightness Control*/
			status = HandleBrightnessControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case PU_CONTRAST_CONTROL: /*Contrast Control*/
			status = HandleContrastControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case PU_SATURATION_CONTROL: /*Saturation Control*/
			status = HandleSaturationControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case PU_GAMMA_CONTROL:
			status = HandleGammaControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL: /*Auto White Balance Temperature Mode Control*/
			status = HandleAWBTempControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		case PU_WHITE_BALANCE_TEMPERATURE_CONTROL: /*White Balance Manual Temperature Control*/
			status = HandleMWBTempControlReq(bRequest, CY_U3P_GET_LSB(wIndex));
			break;
		default:
			glerrorstatus = VC_ERROR_CODE_INVALID_CONTROL;
			break;
		}
	}
	else if(CY_U3P_GET_MSB(wIndex) == FX3_UVC_VC_EXTENSION_UNIT_ID) /*Video Control- Extension Unit*/
	{
		UvcAppDebugPrint(4,"\n\r\n\rVC XU:T ID 0x%x CS 0x%x Req 0x%x ", CY_U3P_GET_MSB(wIndex), wValue, bRequest);

		switch (CY_U3P_GET_MSB(wValue))
		{
		case FX3_UVC_XU_I2C_WRITE_CONTROL:
			status = HandleXUI2CWriteControlReq(bRequest);
			break;
		case FX3_UVC_XU_I2C_READ_CONTROL:
			status = HandleXUI2CReadControlReq(bRequest);
			break;
		}
	}
	else if((CY_U3P_GET_MSB(wIndex) == FX3_UVC_VC_DESCRIPTOR_UNDEFINED) /*Video Control- Undefined ID*/)
	{
		UvcAppDebugPrint(4,"\n\rVC:U ID 0x%x CS 0x%x Req 0x%x ", (wIndex), wValue, bRequest);

		switch (CY_U3P_GET_MSB(wValue))
		{
		case VC_REQUEST_ERROR_CODE_CONTROL:
			status = HandleErrorCodeReq(bRequest, CY_U3P_GET_LSB(wIndex));
			glerrorstatus = VC_ERROR_CODE_NO_ERROR;
			break;
		}
	}
	return status;
}

#endif
