/*
 ## Cypress FX3 Core Library Header (cyFX3_videocontrol.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

#ifndef CYFX3_VIDEOCONTROL_H_
#define CYFX3_VIDEOCONTROL_H_

#define Video_Control_Interface

/* REad/Write Indexes */
#define FX3_UVC_VC_REG_WRITE                      (1)
#define FX3_UVC_VC_REG_READ                       (0)

/* Macros for various UVC parameters */
#define FX3_UVC_VC_LEN_VALUE_BYTE                 (1)
#define FX3_UVC_VC_LEN_VALUE_WORD                 (2)
#define FX3_UVC_VC_LEN_VALUE_DWORD                (4)
#define FX3_UVC_VC_ROI_GET_LEN_VALUE              (10)
#define FX3_UVC_VC_AE_AUTO_MODE_VALUE             (2)
#define FX3_UVC_VC_AE_MANUAL_MODE_VALUE           (4)
#define FX3_UVC_VC_AF_AUTO_MODE_VALUE             (1)
#define FX3_UVC_VC_AF_MANUAL_MODE_VALUE           (0)
#define FX3_UVC_VC_AWB_AUTO_MODE_VALUE            (1)
#define FX3_UVC_VC_AWB_MANUAL_MODE_VALUE          (0)
#define FX3_UVC_VC_BACKLIGHT_COMP_ON_VALUE        (1)
#define FX3_UVC_VC_BACKLIGHT_COMP_OFF_VALUE       (0)

/* Macros for various XU for Video control */
#define FX3_UVC_XU_I2C_WRITE_CONTROL              (0x01)
#define FX3_UVC_XU_I2C_READ_CONTROL               (0x02)

/* Definitions for XU controls */
#define FX3_UVC_XU_I2C_CONTROL_LEN                (12)

/***********************Video Control Structures/Enums*******************/
/* Enum to store the ISP Category register Read API Data Byte indexes*/
typedef enum vcFlickerModeVal
{
  FX3_UVC_VC_FLICKER_MODE_DISABLED = 0,
  FX3_UVC_VC_FLICKER_MODE_50HZ,
  FX3_UVC_VC_FLICKER_MODE_60HZ,
  FX3_UVC_VC_FLICKER_MODE_AUTO
} vcFlickerModeVal_t;

/* Enum to store different error codes apart from the ones listed in
 * CyU3PErrorCode_t in cyu3error.h file S*/
typedef enum CyFx3ExtendedErrorCode
{
  ISP_ERROR_BAD_INTR_FACTOR_READ = 0x80,
  ISP_ERROR_BAD_LOG_LENGTH_VAL,
  ISP_ERROR_BAD_FW_START_INTR_VAL,
  ISP_ERROR_VC_OP_TIMEOUT,
  ISP_ERROR_VC_OP_FAILED,
  ISP_ERROR_UNKNOWN,

  FX3_UVC_XU_DEV_PRO_PREV_SET_CUR = 0x90,

  SPI_PRIMARY_PAGE_CHECKSUM_ERROR = 0xA0,
  SPI_BACKUP_PAGE_CHECKSUM_ERROR,
  SPI_FLASH_BLANK_PAGES_NEED_UPDATE,

} CyFx3ExtendedErrorCode_t;

#pragma pack(push,1)

typedef struct xuSensIspRegRdWrData
{
  uint8_t  sensIspAddr;
  uint32_t regAddr;
  uint32_t regData;
  uint8_t  regLen;
  uint8_t  dataLen;
  uint8_t  opStatus;
}xuSensIspRegRdWrData_t;

#pragma pack(pop)

typedef struct VideoControl_xuI2cWriteRegData
{
  xuSensIspRegRdWrData_t CurrentVal; /*Current Value of the feature*/
  xuSensIspRegRdWrData_t MinVal;     /*Minimum Value supported by the feature*/
  xuSensIspRegRdWrData_t MaxVal;     /*Maximum Value supported by the feature*/
  xuSensIspRegRdWrData_t Res;        /*Resolution supported by the feature*/
  xuSensIspRegRdWrData_t DefaultVal; /*Default Value of the feature*/
  xuSensIspRegRdWrData_t SetVal;     /*Set Value sent by the host application*/
} VideoControl_xuSensIspRegRdWrData_t;



typedef struct VideoControl_uint32
{
  uint32_t CurrentVal;   /*Current Value of the feature*/
  uint32_t MinVal;       /*Minimum Value supported by the feature*/
  uint32_t MaxVal;       /*Maximum Value supported by the feature*/
  uint32_t Res;          /*Resolution supported by the feature*/
  uint32_t DefaultVal;   /*Default Value of the feature*/
  uint32_t SetVal;       /*Set Value sent by the host application*/
} VideoControl_uint32_t;

typedef struct VideoControl_uint16
{
  uint16_t CurrentVal;   /*Current Value of the feature*/
  uint16_t MinVal;       /*Minimum Value supported by the feature*/
  uint16_t MaxVal;       /*Maximum Value supported by the feature*/
  uint16_t Res;          /*Resolution supported by the feature*/
  uint16_t DefaultVal;   /*Default Value of the feature*/
  uint16_t SetVal;       /*Set Value sent by the host application*/
} VideoControl_uint16_t;


typedef struct VideoControl_uint8
{
  uint8_t CurrentVal; /*Current Value of the feature*/
  uint8_t MinVal;     /*Minimum Value supported by the feature*/
  uint8_t MaxVal;     /*Maximum Value supported by the feature*/
  uint8_t Res;        /*Resolution supported by the feature*/
  uint8_t DefaultVal; /*Default Value of the feature*/
  uint8_t SetVal;     /*Set Value sent by the host application*/
} VideoControl_uint8_t;

typedef struct VideoControl_int32
{
  int32_t CurrentVal; /*Current Value of the feature*/
  int32_t MinVal;     /*Minimum Value supported by the feature*/
  int32_t MaxVal;     /*Maximum Value supported by the feature*/
  int32_t Res;        /*Resolution supported by the feature*/
  int32_t DefaultVal; /*Default Value of the feature*/
  int32_t SetVal;     /*Set Value sent by the host application*/
} VideoControl_int32_t;

typedef struct VideoControl_int16
{
  int16_t CurrentVal; /*Current Value of the feature*/
  int16_t MinVal;     /*Minimum Value supported by the feature*/
  int16_t MaxVal;     /*Maximum Value supported by the feature*/
  int16_t Res;        /*Resolution supported by the feature*/
  int16_t DefaultVal; /*Default Value of the feature*/
  int16_t SetVal;     /*Set Value sent by the host application*/
} VideoControl_int16_t;

typedef struct VideoControl_int8
{
  int8_t CurrentVal;   /*Current Value of the feature*/
  int8_t MinVal;       /*Minimum Value supported by the feature*/
  int8_t MaxVal;       /*Maximum Value supported by the feature*/
  int8_t Res;          /*Resolution supported by the feature*/
  int8_t DefaultVal;   /*Default Value of the feature*/
  int8_t SetVal;       /*Set Value sent by the host application*/
} VideoControl_int8_t;

/************************Video Control Global variables************************/
/* Legacy Auto focus Control for rear camera*/
extern VideoControl_uint8_t autoFocusRearCam;

/*******************************Video Control API *****************************/
/*Function to handle all supported UVC Video Control requests*/
extern CyU3PReturnStatus_t CyFx3UvcHandleVCInterfaceReq(uint16_t wIndex, uint16_t wValue, uint16_t bRequest);

#endif /* CYFX3_VIDEOCONTROL_H_ */
