/*
 ## Cypress FX3 Firmware Example Source (CyFx3_videostreaming.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2013-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

#include "cyu3os.h"
#include "cyu3types.h"
#include "cyu3system.h"
#include "cyu3error.h"
#include "cyu3usb.h"
#include "cyfx3_uvc.h"
#include "cyu3_isp.h"
#include "cyfx3_videostreaming.h"




extern uint16_t glFrameTimerPeriod;

/* Variable to monitor the Frame Size of the YUV resolution being transferred. */
extern uint32_t glFrameSizeExpected;






/*Returns the pointer to the Probe Control structure for the corresponding frame index.*/
uint8_t*
CyFx3UvcAppGetVideoProbeCtrlStruct(
		CyU3PUSBSpeed_t usbConType,
		uint8_t         formatIndex,
		uint8_t         frameIndex,
		uint32_t        frameInterval
)
{
	if (CyU3PUsbGetSpeed() == CY_U3P_SUPER_SPEED)
	{
		if (formatIndex == SS_UYVY_FORMAT_INDEX)
		{
			switch (frameIndex)
			{
			case SS_UYVY_1920x1080_INDEX:
				{
				return (uint8_t*)glProbeCtrl_1920x1080_60fps;
				}
			break;

			case SS_UYVY_1280x720_INDEX:
				{
				return (uint8_t*)glProbeCtrl_1280x720_60fps;
				}
			break;

			case SS_UYVY_640x640_INDEX:
				{
				return (uint8_t*)glProbeCtrl_640x640_60fps;
				}
			break;

			case SS_UYVY_640x480_INDEX:
				{
				return (uint8_t*)glProbeCtrl_640x480_60fps;
				}
			break;

			case SS_UYVY_640x360_INDEX:
				{
				return (uint8_t*)glProbeCtrl_640x360_60fps;
				}
			break;

			}

		}
	}
	else if (CyU3PUsbGetSpeed() == CY_U3P_HIGH_SPEED)
	{
		if (formatIndex == HS_UYVY_FORMAT_INDEX)
		{
			switch (frameIndex)
			{
			case HS_UYVY_640x480_INDEX:
				{
					return (uint8_t*)glProbeCtrl_640x480_60fps_HS;
				}
			break;
			}
		}
	}
	return (uint8_t*)glProbeCtrl;
}

#ifdef STILL_CAPTURE_ENABLE

/*	Set the still image resolutions through this function. This function lists all the
 *  supported resolutions in SuperSpeed and HighSpeed. The frame index of resolutions
 *  supported in Still Capture can be different from the frame index of resolutions supported
 *  in Video streaming.
 */
void
CyFx3UvcAppSetStillResolution(
		uint8_t format_index,
		uint8_t resolution_index )
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	glFrameSizeExpected = 0;
    CyU3PDebugPrint(6, "\n\rSet Still Resolution registers");


	if (format_index == SS_UYVY_FORMAT_INDEX)
	{
		switch(resolution_index)
		{
			case SS_UYVY_1920x1080_INDEX:
				status = CyFx3_Isp_Set1920x1080_60fps();
                CyU3PDebugPrint(10, "\n\r%d Status %d",__LINE__,status);

				break;
			case SS_UYVY_640x360_INDEX:
				status = CyFx3_Isp_Set640x360_60fps();
			    CyU3PDebugPrint(10, "\n\r%d Status %d",__LINE__,status);

			    break;

			default:
				break;

		}

		glFrameTimerPeriod = 500;
	}

}
#endif

/*	Set the video resolution through this function. This function lists all the
 *  supported resolutions in SuperSpeed and HighSpeed. The frame index of resolutions
 *  supported in Still Capture can be different from the frame index of resolutions supported
 *  in Video streaming.
 */
void
CyFx3UvcAppSetVideoResolution(
		uint8_t format_index,
		uint8_t resolution_index,
		uint32_t frame_interval,
		CyBool_t isRearCam
)
{

	CyU3PDebugPrint(8,"\r\nEntering UVC  Set video resolution");
	CyU3PDebugPrint(8,"\n\rEntering Set Video Resolution format index = %d", format_index );
	CyU3PDebugPrint(8,"\n\rEntering Set Video Resolution resolution index = %d", resolution_index );
	CyU3PDebugPrint(8,"\n\rEntering Set Video Resolution frame interval = %d", frame_interval );
	CyU3PDebugPrint(8,"\n\rEntering Set Video Resolution Rear Cam = %d", isRearCam );
	glFrameSizeExpected = 0;
	if (CyU3PUsbGetSpeed() == CY_U3P_SUPER_SPEED)
	{
		if(format_index == SS_UYVY_FORMAT_INDEX)
		{
			switch (resolution_index)
			{
			case SS_UYVY_1920x1080_INDEX:
				CyFx3_Isp_Set1920x1080_60fps();
				break;
			case SS_UYVY_1280x720_INDEX:
				CyFx3_Isp_Set1280x720_60fps();
				break;
			case SS_UYVY_640x640_INDEX:
				CyFx3_Isp_Set640x640_60fps();
				break;
			case SS_UYVY_640x480_INDEX:
				CyFx3_Isp_Set640x480_60fps();
				break;
			case SS_UYVY_640x360_INDEX:
				CyFx3_Isp_Set640x360_60fps();
				break;

			default:
				break;

			}
		}

	}
	else if (CyU3PUsbGetSpeed() == CY_U3P_HIGH_SPEED)
	{
		if(format_index == SS_UYVY_FORMAT_INDEX)
		{
			switch (resolution_index)
			{
			case HS_UYVY_640x480_INDEX:
				CyFx3_Isp_Set640x480_30fps();
				break;
			default:
				break;
			}
		}
	}


    glFrameTimerPeriod = (uint16_t) (frame_interval>>10);
    CyU3PDebugPrint(8, "\n\rFrame timer set at =  %d MilliSeconds",glFrameTimerPeriod  );
	glFrameSizeExpected = 1920*1080*2;
	return;
}


