/*
 ## Cypress FX3 Core Library Header (CyFx3_videostreaming.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

#ifndef CyFx3_VIDEOSTREAMING_H_
#define CyFx3_VIDEOSTREAMING_H_
#include "cyu3usb.h"
#include "cyfx3_uvc.h"
#include "cyu3mipicsi.h"




/* List of formats all the supported */
typedef enum CyFx3UYVYVideoFormats2
{
  SS_UYVY_FORMAT_INDEX = 1,
  SS_MJPEG_FORMAT_INDEX,
  SS_NV12_FORMAT_INDEX,
  SS_ARAW_FORMAT_INDEX
} CyFx3UYVYVideoFormats2_t;

typedef enum CyFx3UYVYVideoFormats2USB2
{
  HS_UYVY_FORMAT_INDEX = 1,
  HS_MJPEG_FORMAT_INDEX,
  HS_NV12_FORMAT_INDEX,
  HS_ARAW_FORMAT_INDEX
} CyFx3UYVYVideoFormats2USB2_t;



/*************************************UYVY Resolution************************************/
/* List of all the common UYVY resolutions. */
typedef enum CyFx3SSUYVYVideoRes_t
{
	SS_UYVY_1920x1080_INDEX=1,
	SS_UYVY_1280x720_INDEX,
	SS_UYVY_640x480_INDEX,
	SS_UYVY_640x640_INDEX,
	SS_UYVY_640x360_INDEX,

} CyFx3SSUYVYVideoRes;

typedef enum CyFx3HSUYVYVideoRes_t
{
	HS_UYVY_640x480_INDEX=1,

} CyFx3HSUYVYVideoRes;


/* The following line contains the frame index of the optimal UYVY resolutions in USB3 mode */
#define SS_UYVY_OPTIMUM_FRAME_INDEX       1


#define SS_UYVY_OPTIMUM_FRAME_INDEX    1
#define HS_UYVY_OPTIMUM_FRAME_INDEX    1

#define MAX_FPS_PER_RESN 2

/* The following line lists the total number of UYVY resolutions */
#define SS_UYVY_TOTAL_NO_OF_RES        5
#define HS_UYVY_TOTAL_NO_OF_RES        1

#define HS_MJPEG_TOTAL_NO_OF_RES       0

/* The following line lists the total number of UYVY still image resolutions */
#define SS_UYVY_STILL_TOTAL_NO_OF_RES     2

#define HS_UYVY_STILL_TOTAL_NO_OF_RES     1


/*********************************************************************************************************************************
 *********************************** SUPER SPEED/ HIGH SPEED USB Descriptors Sizes **********************************************************
 *********************************************************************************************************************************/

/* Size of Still Capture uncompressed UYVY resolution */
#define SS_UYVY_STILL_DESCR_SIZE        (SS_UYVY_STILL_TOTAL_NO_OF_RES*4 + 7)

#define HS_UYVY_STILL_DESCR_SIZE        (HS_UYVY_STILL_TOTAL_NO_OF_RES*4 + 7)

/* Total size of Class specific VS descr for front camera =
 *                                 Class specific video streaming input header + Class specific uncompressed VS descr
 *                               + Class specific compressed VS descr + Still Capture uncompressed + Still Capture compressed + Color matching descriptor
 *                               + total number of frame descriptor + 20 bytes (for UYVY 1080p 15fps, 720p 15fps & 60fps, 480p 15fps, 360p 15fps) */
#ifdef USE_ISOCHRONOUS_ENDPOINT
#define SS_TOTAL_SIZE_CLASS_DSCR        (0x0E + 0x1B + SS_UYVY_STILL_DESCR_SIZE + 0x06 +\
                                        (SS_UYVY_TOTAL_NO_OF_RES * 0x1E) + 0x09)
#else
#define SS_TOTAL_SIZE_CLASS_DSCR        (0x0E + 0x1B + SS_UYVY_STILL_DESCR_SIZE + 0x06 +\
                                        (SS_UYVY_TOTAL_NO_OF_RES * 0x1E))
#endif
#define HS_TOTAL_SIZE_CLASS_DSCR        (0x0E + 0x1B + HS_UYVY_STILL_DESCR_SIZE +\
                                        (HS_UYVY_TOTAL_NO_OF_RES * 0x1E))



/* Low byte - Total size of Class specific VS descr for front camera*/
#define SS_TOTAL_SIZE_CLASS_DSCR_L      CY_U3P_GET_LSB(SS_TOTAL_SIZE_CLASS_DSCR)
/* High byte - Total size of Class specific VS descr for front camera*/
#define SS_TOTAL_SIZE_CLASS_DSCR_H      CY_U3P_GET_MSB(SS_TOTAL_SIZE_CLASS_DSCR)

/*for HS*/
/* Low byte - Total size of Class specific VS descr for front camera*/
#define HS_TOTAL_SIZE_CLASS_DSCR_L      CY_U3P_GET_LSB(HS_TOTAL_SIZE_CLASS_DSCR)
/* High byte - Total size of Class specific VS descr for front camera*/
#define HS_TOTAL_SIZE_CLASS_DSCR_H      CY_U3P_GET_MSB(HS_TOTAL_SIZE_CLASS_DSCR)


/* Total size of the device descriptor.
 * Note: The hex values indicates size of the device descriptor along with the sub-descriptors */
  /* Length of this descriptor and all sub descriptors *
   * 0x09 - configuration descriptor size
   * 0x8A/0x7E - sum of all descriptors for a UVC camera other than video streaming class specific desc
   * TOTAL_SIZE_CLASS_DSCR - Video Streaming class specific descriptor
   * +0x09 (per camera for ISOC implementation)
   */

#define	SS_TOTAL_SIZE_CONFIG_DSCR        (0x09 + 0x8A + SS_TOTAL_SIZE_CLASS_DSCR)

#define	HS_TOTAL_SIZE_CONFIG_DSCR        (0x09 + 0x7E + HS_TOTAL_SIZE_CLASS_DSCR)


/* Low byte - Total size of the device descriptor */
#define SS_TOTAL_SIZE_CONFIG_DSCR_L          CY_U3P_GET_LSB(SS_TOTAL_SIZE_CONFIG_DSCR)
/* High byte - Total size of the device descriptor */
#define SS_TOTAL_SIZE_CONFIG_DSCR_H          CY_U3P_GET_MSB(SS_TOTAL_SIZE_CONFIG_DSCR)

/*Low byte - Total size of the HS device descriptor*/
#define HS_TOTAL_SIZE_CONFIG_DSCR_L          CY_U3P_GET_LSB(HS_TOTAL_SIZE_CONFIG_DSCR)
/* High byte - Total size of the device descriptor */
#define HS_TOTAL_SIZE_CONFIG_DSCR_H          CY_U3P_GET_MSB(HS_TOTAL_SIZE_CONFIG_DSCR)



/* YUV Format - Resolution Frame Sizes. */
#define YUV_640x480_FRAME_SIZE			(640*480*2)
#define YUV_640x640_FRAME_SIZE			(640*640*2)
#define YUV_1280x720_FRAME_SIZE			(1280*720*2)
#define YUV_1920x1080_FRAME_SIZE		(1920*1080*2)
#define YUV_640x360_FRAME_SIZE			(640*360*2)



/*Returns the pointer to the Probe Control structure for the corresponding frame index.*/
extern uint8_t *
CyFx3UvcAppGetVideoProbeCtrlStruct (
CyU3PUSBSpeed_t usbConType,
uint8_t         formatIndex,
uint8_t         frameIndex,
uint32_t        frameInterval
);

/*	Set the still image resolutions through this function. This function lists all the
 *  supported resolutions in SuperSpeed and HighSpeed. The frame index of resolutions
 *  supported in Still Capture can be different from the frame index of resolutions supported
 *  in Video streaming.
 */
extern void
CyFx3UvcAppSetStillResolution (
uint8_t format_index, uint8_t resolution_index
);

/*	Set the video resolution through this function. This function lists all the
 *  supported resolutions in SuperSpeed and HighSpeed. The frame index of resolutions
 *  supported in Still Capture can be different from the frame index of resolutions supported
 *  in Video streaming.
 */
extern void
CyFx3UvcAppSetVideoResolution(
uint8_t format_index, uint8_t resolution_index, uint32_t frame_interval, CyBool_t isRearCam
);




#endif /* CyFx3_VIDEOSTREAMING_H_ */
