/*
 ## Cypress FX3 Firmware Example Source (cyu3_isp.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2013-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

#include <cyu3error.h>
#include <cyu3i2c.h>
#include <cyu3utils.h>
#include <cyu3gpio.h>
#include <cyu3usb.h>
#include "cyu3spi.h"
#include "cyfx3_videocontrol.h"
#include "cyu3_isp.h"
#include "cyfx3_uvc.h"
#include "cyfx3_videostreaming.h"
#include "stddef.h"
#include "string.h"

/**************************Variables and structures*****************************/
/* Global variable to display colour bar from ISP*/
CyBool_t glIspDisplayColourBar = CyFalse;

/* Variable to store the ISP firmware Version structure*/
uint8_t glIspFwVer[ISP_FW_VER_SIZE] = {0};

/*Variable to set the video resolutions*/
xuSensIspRegRdWrData_t *sensIspI2CWriteCtrl = {0};

//CyIspCategoryReg_t glSensorConn             = {0x00, 0x15, {0}, 0x1}; /* Used in uvc.c file to check sesor connection*/

CyIspCategoryReg_t brightness           = {0x00, 0x00000064, 0x04};
CyIspCategoryReg_t manualExposure       = {0x04, 0x00000064, 0x04};
CyIspCategoryReg_t autoExposure_start   = {0x90, 0x00000001, 0x04};
CyIspCategoryReg_t autoExposure         = {0xB0, 0x00000001, 0x04};

CyIspCategoryReg_t autoWhiteBal         = {0xA4, 0x00000001, 0x04};
CyIspCategoryReg_t manWhiteBal          = {0xA8, 0x00000000, 0x04};
CyIspCategoryReg_t mwbRedChannel        = {0x60, 0x00000000, 0x04};
CyIspCategoryReg_t mwbGreenChannel      = {0x64, 0x00000000, 0x04};
CyIspCategoryReg_t mwbBlueChannel       = {0x68, 0x00000000, 0x04};
CyIspCategoryReg_t contrast             = {0xFC, 0x00000075, 0x04};
CyIspCategoryReg_t gammaReg             = {0x70, 0x0000003C, 0x04};
CyIspCategoryReg_t saturation           = {0x74, 0x00000082, 0x04};


/*************************** Added by savj **************************************/
/* Generic sensor/ISP I2C write function */
CyU3PReturnStatus_t CyFx3_Sensor_Isp_Reg_I2c_Write(xuSensIspRegRdWrData_t *sensIspI2CWriteCtrl)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t cnt = 0;

	preamble.buffer[0] = sensIspI2CWriteCtrl->sensIspAddr; /* Slave address: write operation */

	if ((sensIspI2CWriteCtrl->sensIspAddr ==ISP_SLAVE_ADDR_1)||(sensIspI2CWriteCtrl->sensIspAddr ==ISP_SLAVE_ADDR_2))
	{
		preamble.buffer[1] = CY_U3P_GET_LSB(sensIspI2CWriteCtrl->regAddr);
	}
	else if (sensIspI2CWriteCtrl->sensIspAddr ==SENSOR_SLAVE_ADDR)
	{
		preamble.buffer[1] = CY_U3P_GET_MSB(sensIspI2CWriteCtrl->regAddr);
		preamble.buffer[2] = CY_U3P_GET_LSB(sensIspI2CWriteCtrl->regAddr);
	}

	preamble.length = 1 + sensIspI2CWriteCtrl->regLen;
	preamble.ctrlMask = 0x0000;

	for (cnt = 0; cnt < 3; cnt++)
	{
		status = CyU3PI2cTransmitBytes(&preamble, (uint8_t*)&sensIspI2CWriteCtrl->regData, sensIspI2CWriteCtrl->dataLen, 10);
		if (status == CY_U3P_SUCCESS)
		{
			break;
		}
	}

	return status;
}

CyU3PReturnStatus_t CyFx3_Sensor_Isp_Reg_I2c_Read(xuSensIspRegRdWrData_t *ispI2CReadCtrl)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t cnt = 0;

	preamble.buffer[0] = ispI2CReadCtrl->sensIspAddr; /* Slave address: write operation */

	if ((ispI2CReadCtrl->sensIspAddr ==ISP_SLAVE_ADDR_1)||(ispI2CReadCtrl->sensIspAddr ==ISP_SLAVE_ADDR_2))
	{
		preamble.buffer[1] = CY_U3P_GET_LSB(ispI2CReadCtrl->regAddr);
		preamble.buffer[2] = ispI2CReadCtrl->sensIspAddr + 1; /* Slave address: read operation */

		preamble.length = 3;
		preamble.ctrlMask = 0x0002;
	}
	else if (ispI2CReadCtrl->sensIspAddr ==SENSOR_SLAVE_ADDR)
	{
		preamble.buffer[1] = CY_U3P_GET_MSB(ispI2CReadCtrl->regAddr);
		preamble.buffer[2] = CY_U3P_GET_LSB(ispI2CReadCtrl->regAddr);
		preamble.buffer[3] = ispI2CReadCtrl->sensIspAddr + 1; /* Slave address: read operation */

		preamble.length = 4;
		preamble.ctrlMask = 0x0004;
	}

	for (cnt = 0; cnt < 3; cnt++)
	{
		status = CyU3PI2cReceiveBytes(&preamble, (uint8_t*)&ispI2CReadCtrl->regData, ispI2CReadCtrl->dataLen, 10);
		if (status == CY_U3P_SUCCESS)
		{
			break;
		}
	}

	return status;
}

/*I2C Category Write function*/
CyU3PReturnStatus_t CyFx3_Sensor_Reg_I2c_Write(uint8_t SensorAdd, uint16_t RegAdd, uint8_t value)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t cnt = 0;
	uint8_t buffer[4] = {0};

	preamble.buffer[0] = SensorAdd; /* Slave address: write operation */
	preamble.buffer[1] = CY_U3P_GET_MSB(RegAdd);
	preamble.buffer[2] = CY_U3P_GET_LSB(RegAdd);

	preamble.length = 3;
	preamble.ctrlMask = 0x0000;

	buffer[0] = value;

	for (cnt = 0; cnt < 3; cnt++)
	{
		status = CyU3PI2cTransmitBytes(&preamble, buffer, 1, 10);
		if (status == CY_U3P_SUCCESS)
		{
			break;
		}
	}

	return status;
}

CySensorConfig_t sensorConfig[] =
{
		{0x3000, 0x1, },
		{0x3001, 0x0, },
		{0x3002, 0x0, },
		{0x3003, 0x0, },
		{0x3004, 0x0, },
		{0x3004, 0x10,},
		{0x3005, 0x11,}, // shift 2bit , 12 bit
		{0x3005, 0x11,},
		{0x3006, 0x0, },
		{0x3007, 0x10,},
		{0x3007, 0x10,},
		{0x3007, 0x10,},
		{0x3008, 0x0, },
		{0x3008, 0x10,},
		{0x3009, 0x0, }, // 60fps serial
		{0x300a, 0x01,}, //black level 0xf0
		{0x300b, 0x0, },
		{0x300c, 0x0, },
		{0x300c, 0x0, },
		{0x300c, 0x0, },
		{0x300d, 0x0, },
		{0x300d, 0x20,},
		{0x300e, 0x1, },
		{0x300e, 0x1, },
		{0x300e, 0x1, },
		{0x300e, 0x1, },
		{0x300e, 0x1, },
		{0x300f, 0x1, },
		{0x300f, 0x1, },
		{0x300f, 0x1, },
		{0x3010, 0x1, },
		{0x3010, 0x1, },
		{0x3010, 0x1, },
		{0x3010, 0x1, },
		{0x3010, 0x1, },
		{0x3010, 0x21,},
		{0x3010, 0x21,},
		{0x3011, 0x0, },
		{0x3012, 0xf0,},
		{0x3013, 0x0, },
		{0x3014, 0x0, },
		{0x3015, 0x0, },
		{0x3016, 0x0, },
		{0x3016, 0x8, },
		{0x3018, 0x65,}, // VMAX 1080 + 45
		{0x3019, 0x4, },
		{0x301a, 0x0, },
		{0x301b, 0x4C,},  // HMAX 1920 + 280
		{0x301c, 0x04,},//
		{0x301d, 0x26,},
		{0x301e, 0x2, },
		{0x3020, 0x64,},
		{0x3021, 0x2, },
		{0x3022, 0x0, },
		{0x3023, 0x0, },
		{0x3024, 0x0, },
		{0x3025, 0x0, },
		{0x3026, 0x0, },
		{0x3027, 0x0, },
		{0x3028, 0x0, },
		{0x3029, 0x0, },
		{0x302a, 0x0, },
		{0x302b, 0x0, },
		{0x302c, 0x0, },
		{0x302d, 0x0, },
		{0x302e, 0x0, },
		{0x302f, 0x0, },
		{0x3034, 0x0, },
		{0x3036, 0xc, },
		{0x3038, 0x3c,},
		{0x3039, 0x0, },
		{0x303a, 0x50,},
		{0x303b, 0x4, },
		{0x303c, 0x0, },
		{0x303d, 0x0, },
		{0x303e, 0x9c,},
		{0x303f, 0x7, },
		{0x3044, 0x1, },
		{0x3044, 0x1, },
		{0x3044, 0x1, },
        {0x3044, 0xE1,},//31 //////////////////// Setting parallel operation
		{0x3045, 0x1, },
		{0x3045, 0x1, },
		{0x3045, 0x1, },
		{0x3046, 0x0, },
		{0x3046, 0x0, },
		{0x3046, 0x0, },
		{0x3047, 0x0, },
		{0x3047, 0x0, },
		{0x3047, 0x8, },
		{0x3047, 0x8, },
		{0x3047, 0x8, },
		{0x3048, 0x13,},
		{0x3048, 0x13,},
		{0x3048, 0x13,},
		{0x3048, 0x13,},
		{0x3049, 0x2, },
		{0x3049, 0xa, },
		{0x3049, 0xa, },
		{0x3049, 0xa, },
		{0x304a, 0x0, },
		{0x304a, 0x0, },
		{0x304b, 0x0, },
		{0x304b, 0x0, },
		{0x304c, 0x0, },
		{0x304d, 0x0, },
		{0x304e, 0x0, },
		{0x304f, 0x0, },
		{0x3050, 0x6, },
		{0x3050, 0x16,},
		{0x3051, 0x0, },
		{0x3052, 0x1, },
		{0x3053, 0x0, },
		{0x3053, 0x0, },
		{0x3054, 0x1, },
		{0x3054, 0x3, },
		{0x3054, 0x3, },
		{0x3054, 0x23,},
		{0x3054, 0x63,},
		{0x3055, 0x0, },
		{0x3055, 0x0, },
		{0x3056, 0x9, },
		{0x3056, 0xc9,},
		{0x3057, 0x4, },
		{0x3057, 0x64,},
		{0x3058, 0x0, },
		{0x305b, 0x00,},//0x00, // neu
		{0x305c, 0x30,},//0x30, // neu
		{0x305d, 0x08,},//0x08, // neu
		{0x305d, 0x08,},//0x08, // neu
		{0x305d, 0x08,},//0x08, // neu
		{0x305e, 0x30,},//0x30, // neu
		{0x305f, 0x08,},//0x08, // neu
		{0x305f, 0x08,},
		{0x305f, 0x08,},
		{0x3060, 0x0, },
		{0x3060, 0x20,},
		{0x3060, 0xe0,},
		{0x3061, 0x1, },
		{0x3061, 0x1, },
		{0x3062, 0x0, },
		{0x3062, 0x0, },
		{0x3062, 0x0, },
		{0x3062, 0x18,},
		{0x3062, 0x18,},
		{0x3063, 0xe8,},
		{0x3064, 0x0, },
		{0x3064, 0x0, },
		{0x3064, 0x0, },
		{0x3064, 0x0, },
		{0x3064, 0x0, },
		{0x3065, 0x0, },
		{0x3065, 0x0, },
		{0x3065, 0x0, },
		{0x3065, 0x20,},
		{0x3065, 0x20,},
		{0x3066, 0x0, },
		{0x3066, 0x0, },
		{0x3068, 0x0, },
		{0x3068, 0x0, },
		{0x3069, 0x0, },
		{0x3069, 0xc, },
		{0x306a, 0x19,},
		{0x306b, 0x0, },
		{0x306c, 0x0, },
		{0x306d, 0x64,},
		{0x306e, 0x0, },
		{0x306f, 0x0, },
		{0x3070, 0x1, },
		{0x3074, 0x0, },
		{0x3074, 0x0, },
		{0x3075, 0x0, },
		{0x3076, 0x0, },
		{0x3077, 0x0, },
		{0x3077, 0x0, },
		{0x3078, 0x0, },
		{0x3079, 0x0, },
		{0x307a, 0x0, },
		{0x307a, 0x0, },
		{0x307b, 0x0, },
		{0x307b, 0x0, },
		{0x307c, 0x0, },
		{0x307d, 0x0, },
		{0x307e, 0x0, },
		{0x307e, 0x0, },
		{0x307f, 0x0, },
		{0x3080, 0x0, },
		{0x3081, 0x0, },
		{0x3081, 0x0, },
		{0x3084, 0xc, },
		{0x3085, 0xc, },
		{0x3086, 0x1, },
		{0x3086, 0x1, },
		{0x3086, 0x1, },
		{0x3087, 0x0, },
		{0x3088, 0x0, },
		{0x3088, 0x2, },
		{0x3088, 0x2, },
		{0x3089, 0xff,},
		{0x308a, 0x0, },
		{0x308a, 0x0, },
		{0x308c, 0x0, },
		{0x308c, 0x2, },
		{0x308c, 0x2, },
		{0x308d, 0x0, },
		{0x308e, 0x0, },
		{0x308f, 0x0, },
		{0x3090, 0x0, },
		{0x3091, 0x0, },
		{0x3092, 0x0, },
		{0x3093, 0x1, },
		{0x3094, 0x1, },
		{0x3095, 0x1, },
		{0x3096, 0x0, },
		{0x3097, 0x0, },
		{0x3098, 0x0, },
		{0x3099, 0x0, },
		{0x309c, 0x0, },
		{0x309d, 0x0, },
		{0x309d, 0x40,},
		{0x309e, 0x1, },
		{0x309e, 0x71,},
		{0x309f, 0x1, },
		{0x309f, 0x1, },
		{0x309f, 0x5, },
		{0x309f, 0x5, },
		{0x309f, 0x5, },
		{0x309f, 0x5, },
		{0x309f, 0x5, },
		{0x30a0, 0x0, },
		{0x30a1, 0x0, },
		{0x30a1, 0x0, },
		{0x30a1, 0x4, },
		{0x30a1, 0x24,},
		{0x30a1, 0x24,},
		{0x30a2, 0x3, },
		{0x30a2, 0x23,},
		{0x30a3, 0x2, },
		{0x30a3, 0x2, },
		{0x30a4, 0x2, },
		{0x30a4, 0xa, },
		{0x30a4, 0x2a,},
		{0x30a4, 0x2a,},
		{0x30a5, 0x0, },
		{0x30a5, 0x0, },
		{0x30a5, 0x0, },
		{0x30a8, 0x0, },
		{0x30a8, 0x0, },
		{0x30a8, 0x0, },
		{0x30a8, 0x0, },
		{0x30a9, 0x10,},
		{0x30ab, 0x10,},
		{0x30ad, 0x10,},
		{0x30af, 0x10,},
		{0x30b1, 0xff,},
		{0x30b3, 0x1, },
		{0x30b3, 0x1, },
		{0x30b3, 0x41,},
		{0x30b4, 0x0, },
		{0x30b5, 0x0, },
		{0x30b5, 0x20,},
		{0x30b5, 0x60,},
		{0x30b5, 0x60,},
		{0x30b6, 0x0, },
		{0x30b6, 0x2, },
		{0x30b6, 0x2, },
		{0x30b6, 0x82,},
		{0x30b7, 0x3, },
		{0x30b7, 0x3, },
		{0x30b7, 0x13,},
		{0x30b8, 0xf, },
		{0x30b9, 0x0, },
		{0x30b9, 0x0, },
		{0x30b9, 0x0, },
		{0x30b9, 0x0, },
		{0x30b9, 0x0, },
		{0x30b9, 0x0, },
		{0x30ba, 0x0, },
		{0x30be, 0x0, },
		{0x30be, 0x0, },
		{0x30be, 0x0, },
		{0x30be, 0x0, },
		{0x30bf, 0x10,},
		{0x30c0, 0x0, },
		{0x30c1, 0x0, },
		{0x30c1, 0x0, },
		{0x30c1, 0x0, },
		{0x30c1, 0x0, },
		{0x30c4, 0x0, },
		{0x30c4, 0x0, },
		{0x30c5, 0x20,},
		{0x30c6, 0x0, },
		{0x30c6, 0x30,},
		{0x30c7, 0x20,},
		{0x30c8, 0x40,},
		{0x30c9, 0x1, },
		{0x30c9, 0x21,},
		{0x30ca, 0x0, },
		{0x30cc, 0x1, },
		{0x30cc, 0x21,},
		{0x30cc, 0x21,},
		{0x30cd, 0x1e,},
		{0x30ce, 0x0, },
		{0x30cf, 0x1, },
		{0x30cf, 0xd1,},
		{0x30d0, 0x1b,},
		{0x30d1, 0x0, },
		{0x30d2, 0x5f,},
		{0x30d3, 0x0, },
		{0x30d4, 0x0, },
		{0x30d5, 0x0, },
		{0x30d6, 0x0, },
		{0x30d7, 0x0, },
		{0x30d8, 0x0, },
		{0x30d9, 0x0, },
		{0x30da, 0x0, },
		{0x30db, 0x0, },
		{0x30e0, 0x0, },
		{0x30e0, 0x0, },
		{0x30e0, 0x0, },
		{0x30e0, 0x0, },
		{0x30e1, 0xff,},
		{0x30e2, 0xff,},
		{0x30e3, 0xff,},
		{0x30e4, 0xff,},
		{0x30e5, 0xff,},
		{0x30e6, 0xff,},
		{0x30e7, 0x2, },
		{0x30e8, 0x1, },
		{0x30e9, 0x4, },
		{0x30ea, 0x4, },
		{0x30eb, 0x1, },
		{0x30ec, 0x2, },
		{0x30ed, 0x2, },
		{0x30ed, 0x42,},
		{0x30ee, 0x4, },
		{0x30ee, 0x44,},
		{0x30ef, 0x0, },
		{0x30f0, 0xf0,},
		{0x30f1, 0x0, },
		{0x30f2, 0x0, },
		{0x30f3, 0x0, },
		{0x30f4, 0xf0,},
		{0x30f5, 0x0, },
		{0x30f6, 0x0, },
		{0x30f7, 0x0, },
		{0x30f8, 0xf0,},
		{0x30f9, 0x0, },
		{0x30fa, 0x0, },
		{0x30fb, 0x0, },
		{0x3101, 0x0, },
		{0x3101, 0x0, },
		{0x3101, 0x0, },
		{0x3101, 0x0, },
		{0x3102, 0x0, },
		{0x3102, 0x0, },
		{0x3102, 0x0, },
		{0x3102, 0x10,},
		{0x3102, 0x70,},
		{0x3103, 0x33,},
		{0x3104, 0x0, },
		{0x3105, 0x0, },
		{0x3106, 0x0, },
		{0x3107, 0x0, },
		{0x310b, 0x0, },
		{0x310b, 0x0, },
		{0x310b, 0x0, },
		{0x310b, 0x0, },
		{0x310c, 0x0, },
		{0x310c, 0x10,},
		{0x310c, 0x10,},
		{0x310d, 0x7f,},
		{0x310e, 0x7, },
		{0x310f, 0xe, },
		{0x3110, 0x0, },
		{0x3110, 0x0, },
		{0x3111, 0x0, },
		{0x3111, 0x0, },
		{0x3116, 0x0, },
		{0x3116, 0x2, },
		{0x3116, 0x2, },
		{0x3116, 0x2, },
		{0x3117, 0x2, },
		{0x3117, 0x22,},
		{0x3118, 0x2, },
		{0x3118, 0x2, },
		{0x3118, 0x2, },
		{0x3119, 0x2, },
		{0x311d, 0x8, },
		{0x311e, 0x3, },
		{0x311e, 0x3, },
		{0x311f, 0x4, },
		{0x311f, 0x4, },
		{0x3120, 0x0, },
		{0x3120, 0x0, },
		{0x3120, 0x0, },
		{0x3120, 0x0, },
		{0x3120, 0x10,},
		{0x3121, 0x0, },
		{0x3121, 0x0, },
		{0x3121, 0x0, },
		{0x3121, 0x0, },
		{0x3121, 0x0, },
		{0x3121, 0x0, },
		{0x3122, 0x0, },
		{0x3122, 0x0, },
		{0x3122, 0x0, },
		{0x3122, 0x0, },
		{0x3122, 0x0, },
		{0x3123, 0xf, },
		{0x3123, 0xf, },
		{0x3124, 0x3, },
		{0x3124, 0x3, },
		{0x3124, 0x3, },
		{0x3124, 0x3, },
		{0x3125, 0xfe,},
		{0x3126, 0x3, },
		{0x3127, 0xfe,},
		{0x3128, 0x5, },
		{0x312b, 0x0, },
		{0x312b, 0x0, },
		{0x312c, 0xf, },
		{0x3131, 0x1, },
		{0x3131, 0x1, },
		{0x3132, 0x0, },
		{0x3132, 0x0, },
		{0x3133, 0x0, },
		{0x3133, 0x0, },
		{0x3134, 0x6, },
		{0x3139, 0x0, },
		{0x3139, 0x0, },
		{0x313d, 0x0, },
		{0x313d, 0x0, },
		{0x313e, 0x1, },
		{0x313f, 0x0, },
		{0x3140, 0x0, },
		{0x3141, 0x0, },
		{0x3142, 0x1, },
		{0x3142, 0x1, },
		{0x3142, 0x51,},
		{0x3146, 0x2, },
		{0x3146, 0x2, },
		{0x3147, 0x41,},
		{0x3148, 0xf, },
		{0x3148, 0xf, },
		{0x3154, 0x0, },
		{0x3155, 0xb, },
		{0x3156, 0x0, },
		{0x3156, 0x60,},
		{0x3157, 0x1, },
		{0x3158, 0x21,},
		{0x3159, 0x0, },
		{0x315c, 0x2, },
		{0x315c, 0x12,},
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315d, 0x0, },
		{0x315e, 0x0, },
		{0x315f, 0x2, },
		{0x315f, 0x12,},
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3160, 0x0, },
		{0x3161, 0x0, },
		{0x3163, 0x0, },
		{0x3163, 0x10,},
		{0x3165, 0x0, },
		{0x3165, 0x0, },
		{0x3165, 0x0, },
		{0x3165, 0x0, },
		{0x3166, 0x80,},
		{0x316b, 0x0, },
		{0x316b, 0x0, },
		{0x316b, 0x0, },
		{0x316c, 0x0, },
		{0x316c, 0x0, },
		{0x316c, 0x0, },
		{0x316c, 0x0, },
		{0x316c, 0x0, },
		{0x316d, 0x0, },
		{0x316e, 0x0, },
		{0x316e, 0x0, },
		{0x316f, 0xff,},
		{0x3170, 0x1, },
		{0x3171, 0x2, },
		{0x3171, 0x2, },
		{0x3173, 0x0, },
		{0x3173, 0x0, },
		{0x3173, 0x0, },
		{0x3173, 0x0, },
		{0x3174, 0x1, },
		{0x3174, 0x5, },
		{0x3174, 0x5, },
		{0x3175, 0x0, },
		{0x3175, 0x0, },
		{0x3175, 0x0, },
		{0x3175, 0x0, },
		{0x3176, 0x0, },
		{0x3177, 0x0, },
		{0x3177, 0x0, },
		{0x3178, 0x0, },
		{0x3179, 0x0, },
		{0x3179, 0x0, },
		{0x317b, 0x0, },
		{0x317d, 0xc, },
		{0x317e, 0x0, },
		{0x317f, 0x0, },
		{0x317f, 0x0, },
		{0x3180, 0x0, },
		{0x3183, 0x4, },
		{0x3183, 0x24,},
		{0x3184, 0x3, },
		{0x3185, 0x0, },
		{0x3186, 0xe, },
		{0x3187, 0x0, },
		{0x3188, 0x27,},
		{0x3189, 0x0, },
		{0x318a, 0xee,},
		{0x318b, 0x4, },
		{0x318c, 0xed,},
		{0x318d, 0x4, },
		{0x318e, 0xee,},
		{0x318f, 0x4, },
		{0x3190, 0x4, },
		{0x3191, 0xf6,},
		{0x3192, 0x0, },
		{0x3193, 0x2, },
		{0x3194, 0x4, },
		{0x3195, 0x14,},
		{0x3196, 0x1c,},
		{0x3197, 0x0, },
		{0x3198, 0x3c,},
		{0x3199, 0x0, },
		{0x319a, 0xd2,},
		{0x319b, 0x7, },
		{0x319b, 0x7, },
		{0x319c, 0x7d,},
		{0x319d, 0xc, },
		{0x319e, 0x5, },
		{0x319e, 0xa5,},
		{0x319f, 0x50,},
		{0x31a1, 0x0, },
		{0x31a1, 0x30,},
		{0x31a2, 0x3, },
		{0x31a3, 0x9, },
		{0x31a4, 0x0, },
		{0x31a5, 0xef,},
		{0x31a6, 0x1, },
		{0x31a7, 0xf3,},
		{0x31a8, 0x1, },
		{0x31aa, 0x5, },
		{0x31ab, 0x4, },
		{0x31ac, 0x0, },
		{0x31ad, 0x50,},
		{0x31af, 0x0, },
		{0x31af, 0x2, },
		{0x31af, 0x2, },
		{0x31b0, 0x1, },
		{0x31b0, 0x1, },
		{0x31b1, 0x0, },
		{0x31b1, 0x0, },
		{0x31b2, 0x0, },
		{0x31b2, 0x0, },
		{0x31b3, 0x0, },
		{0x31b3, 0x0, },
		{0x31b4, 0x0, },
		{0x31b5, 0x0, },
		{0x31b5, 0x0, },
		{0x31b6, 0x0, },
		{0x31b6, 0x0, },
		{0x31b7, 0x0, },
		{0x31b7, 0x0, },
		{0x31b8, 0x0, },
		{0x31b8, 0x0, },
		{0x31b9, 0x0, },
		{0x31b9, 0x0, },
		{0x31ba, 0x0, },
		{0x31ba, 0x0, },
		{0x31bb, 0x0, },
		{0x31bb, 0x0, },
		{0x31bc, 0x14,},
		{0x31bd, 0x5, },
		{0x31bd, 0x25,},
		{0x31be, 0x51,},
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31bf, 0x0, },
		{0x31c0, 0x0, },
		{0x31c0, 0x0, },
		{0x31c0, 0x0, },
		{0x31c0, 0x0, },
		{0x31c0, 0x0, },
		{0x31c1, 0x3f,},
		{0x31c1, 0x3f,},
		{0x31c2, 0x7f,},
		{0x31c3, 0xe, },
		{0x31c4, 0x2, },
		{0x31c4, 0x6, },
		{0x31c4, 0x16,},
		{0x31d8, 0x0, },
		{0x31d9, 0x0, },
		{0x31da, 0x0, },
		{0x31db, 0x0, },
		{0x31dd, 0x0, },
		{0x31e0, 0x48,},
		{0x31e1, 0xee,},
		{0x31e2, 0x48,},
		{0x31e3, 0xee,},
		{0x31e4, 0xee,},
		{0x31e5, 0xee,},
		{0x31e6, 0xee,},
		{0x31e7, 0xee,},
		{0x31e8, 0x4a,},
		{0x31e9, 0xdf,},
		{0x31ea, 0x4a,},
		{0x31eb, 0xdf,},
		{0x31ec, 0xdf,},
		{0x31ed, 0xdf,},
		{0x31ee, 0xdf,},
		{0x31ef, 0xdf,},
		{0x3202, 0x0, },
		{0x3203, 0x4, },
		{0x3204, 0x1, },
		{0x3205, 0x42,},
		{0x3206, 0x0, },
		{0x3207, 0xda,},
		{0x3208, 0x1, },
		{0x3209, 0x44,},
		{0x320a, 0x1, },
		{0x320b, 0x0, },
		{0x320c, 0x0, },
		{0x320d, 0x0, },
		{0x320e, 0x0, },
		{0x320f, 0x1, },
		{0x3210, 0x0, },
		{0x3211, 0x4, },
		{0x3212, 0x2, },
		{0x3212, 0x32,},
		{0x3213, 0x2, },
		{0x3214, 0x0, },
		{0x3214, 0x30,},
		{0x3215, 0x0, },
		{0x3216, 0x0, },
		{0x3216, 0x0, },
		{0x3216, 0x0, },
		{0x3217, 0x9, },
		{0x3218, 0x0, },
		{0x3219, 0xff,},
		{0x321a, 0xf, },
		{0x321b, 0x4, },
		{0x321c, 0xf, },
		{0x321d, 0x2, },
		{0x321d, 0x32,},
		{0x321e, 0xa, },
		{0x321f, 0x0, },
		{0x321f, 0xb0,},
		{0x3220, 0x0, },
		{0x3221, 0x0, },
		{0x3221, 0x0, },
		{0x3221, 0x0, },
		{0x3222, 0x5, },
		{0x3223, 0x0, },
		{0x3223, 0x10,},
		{0x3224, 0x0, },
		{0x3225, 0x0, },
		{0x3225, 0x0, },
		{0x3225, 0x0, },
		{0x3226, 0x0, },
		{0x3227, 0x0, },
		{0x3227, 0x0, },
		{0x3228, 0x0, },
		{0x3229, 0x1, },
		{0x3229, 0x3, },
		{0x3229, 0x3, },
		{0x322a, 0x45,},
		{0x322b, 0x0, },
		{0x322b, 0x30,},
		{0x322c, 0x7, },
		{0x322d, 0x15,},
		{0x322e, 0x1, },
		{0x322e, 0xc1,},
		{0x322f, 0x13,},
		{0x3230, 0x0, },
		{0x3230, 0x0, },
		{0x3230, 0x0, },
		{0x3231, 0xeb,},
		{0x3232, 0x1, },
		{0x3232, 0x71,},
		{0x3233, 0x21,},
		{0x3234, 0x0, },
		{0x3234, 0x0, },
		{0x3234, 0x0, },
		{0x3235, 0x15,},
		{0x3236, 0x1, },
		{0x3236, 0x71,},
		{0x3237, 0x13,},
		{0x3238, 0x0, },
		{0x3238, 0x0, },
		{0x3238, 0x0, },
		{0x3239, 0xf1,},
		{0x323a, 0x1, },
		{0x323a, 0x31,},
		{0x323b, 0x21,},
		{0x323c, 0x0, },
		{0x323c, 0x0, },
		{0x323c, 0x0, },
		{0x323d, 0x16,},
		{0x323e, 0x1, },
		{0x323e, 0x61,},
		{0x323f, 0x13,},
		{0x3240, 0x0, },
		{0x3240, 0x0, },
		{0x3240, 0x0, },
		{0x3241, 0xf2,},
		{0x3242, 0x1, },
		{0x3242, 0x21,},
		{0x3243, 0x21,},
		{0x3244, 0x0, },
		{0x3244, 0x0, },
		{0x3244, 0x0, },
		{0x3245, 0x16,},
		{0x3246, 0x1, },
		{0x3246, 0x61,},
		{0x3247, 0x13,},
		{0x3248, 0xf2,},
		{0x3249, 0x1, },
		{0x3249, 0x21,},
		{0x324a, 0x21,},
		{0x324b, 0x0, },
		{0x324b, 0x0, },
		{0x324b, 0x0, },
		{0x324c, 0x37,},
		{0x324d, 0x1, },
		{0x324d, 0xb1,},
		{0x324e, 0x13,},
		{0x324f, 0xe, },
		{0x3250, 0x2, },
		{0x3250, 0x22,},
		{0x3251, 0x21,},
		{0x3252, 0x1, },
		{0x3252, 0x1, },
		{0x3252, 0x1, },
		{0x3253, 0x15,},
		{0x3254, 0x1, },
		{0x3254, 0xb1,},
		{0x3255, 0x1e,},
		{0x3256, 0x0, },
		{0x3256, 0x0, },
		{0x3256, 0x0, },
		{0x3257, 0x3c,},
		{0x3258, 0x1, },
		{0x3258, 0x51,},
		{0x3259, 0x11,},
		{0x325a, 0x0, },
		{0x325a, 0x0, },
		{0x325a, 0x0, },
		{0x325b, 0x44,},
		{0x325c, 0x0, },
		{0x325c, 0x60,},
		{0x325d, 0xd, },
		{0x325e, 0x0, },
		{0x325e, 0x0, },
		{0x325e, 0x0, },
		{0x325f, 0x3, },
		{0x3260, 0x0, },
		{0x3260, 0x0, },
		{0x3261, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3262, 0x0, },
		{0x3263, 0x0, },
		{0x3263, 0x0, },
		{0x3263, 0x0, },
		{0x3263, 0x0, },
		{0x3264, 0xde,},
		{0x3265, 0x0, },
		{0x3265, 0xb0,},
		{0x3266, 0xd, },
		{0x3267, 0x1d,},
		{0x3268, 0x1, },
		{0x3268, 0xa1,},
		{0x3269, 0x11,},
		{0x326a, 0x0, },
		{0x326a, 0x0, },
		{0x326a, 0x0, },
		{0x326b, 0xbc,},
		{0x326c, 0x0, },
		{0x326c, 0x40,},
		{0x326d, 0x4, },
		{0x326e, 0x0, },
		{0x326e, 0x0, },
		{0x326e, 0x0, },
		{0x326f, 0x44,},
		{0x3270, 0x0, },
		{0x3270, 0x90,},
		{0x3271, 0xb, },
		{0x3272, 0x0, },
		{0x3272, 0x0, },
		{0x3272, 0x0, },
		{0x3273, 0x44,},
		{0x3274, 0x0, },
		{0x3274, 0x70,},
		{0x3275, 0xb, },
		{0x3276, 0x0, },
		{0x3276, 0x0, },
		{0x3276, 0x0, },
		{0x3277, 0x44,},
		{0x3278, 0x0, },
		{0x3278, 0x70,},
		{0x3279, 0xb, },
		{0x327a, 0x0, },
		{0x327a, 0x0, },
		{0x327a, 0x0, },
		{0x327b, 0x15,},
		{0x327c, 0x1, },
		{0x327c, 0x71,},
		{0x327d, 0x21,},
		{0x327e, 0x0, },
		{0x327e, 0x0, },
		{0x327e, 0x0, },
		{0x327f, 0x47,},
		{0x3280, 0x0, },
		{0x3280, 0x40,},
		{0x3281, 0x4, },
		{0x3282, 0x0, },
		{0x3282, 0x0, },
		{0x3282, 0x0, },
		{0x3283, 0xd8,},
		{0x3284, 0x0, },
		{0x3284, 0x60,},
		{0x3285, 0xd, },
		{0x3286, 0x0, },
		{0x3286, 0x0, },
		{0x3286, 0x0, },
		{0x3287, 0x18,},
		{0x3288, 0x1, },
		{0x3288, 0x51,},
		{0x3289, 0x11,},
		{0x328a, 0x0, },
		{0x328a, 0x0, },
		{0x328a, 0x0, },
		{0x328b, 0x1, },
		{0x328c, 0x0, },
		{0x328c, 0xc0,},
		{0x328d, 0x7, },
		{0x328e, 0x0, },
		{0x328e, 0x0, },
		{0x328e, 0x0, },
		{0x328f, 0xf2,},
		{0x3290, 0x0, },
		{0x3290, 0x80,},
		{0x3291, 0x10,},
		{0x3292, 0x72,},
		{0x3293, 0x1, },
		{0x3293, 0xe1,},
		{0x3294, 0x1d,},
		{0x3295, 0x0, },
		{0x3295, 0x0, },
		{0x3295, 0x0, },
		{0x3296, 0xfa,},
		{0x3297, 0x0, },
		{0x3297, 0x0, },
		{0x3298, 0x11,},
		{0x3299, 0x7a,},
		{0x329a, 0x1, },
		{0x329a, 0x61,},
		{0x329b, 0x1e,},
		{0x329c, 0x0, },
		{0x329c, 0x0, },
		{0x329c, 0x0, },
		{0x329d, 0xf6,},
		{0x329e, 0x0, },
		{0x329e, 0x40,},
		{0x329f, 0x11,},
		{0x32a0, 0x76,},
		{0x32a1, 0x1, },
		{0x32a1, 0xa1,},
		{0x32a2, 0x1e,},
		{0x32a3, 0x0, },
		{0x32a3, 0x0, },
		{0x32a3, 0x0, },
		{0x32a4, 0x54,},
		{0x32a5, 0x0, },
		{0x32a5, 0xa0,},
		{0x32a6, 0x5, },
		{0x32a7, 0x0, },
		{0x32a7, 0x0, },
		{0x32a7, 0x0, },
		{0x32a8, 0x3c,},
		{0x32a9, 0x0, },
		{0x32a9, 0xc0,},
		{0x32aa, 0x4, },
		{0x32ab, 0x1d,},
		{0x32ac, 0x1, },
		{0x32ac, 0xd1,},
		{0x32ad, 0x12,},
		{0x32ae, 0x0, },
		{0x32ae, 0x0, },
		{0x32ae, 0x0, },
		{0x32af, 0x3d,},
		{0x32b0, 0x0, },
		{0x32b0, 0xd0,},
		{0x32b1, 0x4, },
		{0x32b2, 0x1e,},
		{0x32b3, 0x1, },
		{0x32b3, 0xe1,},
		{0x32b4, 0x12,},
		{0x32b5, 0x0, },
		{0x32b5, 0x0, },
		{0x32b5, 0x0, },
		{0x32b6, 0x44,},
		{0x32b7, 0x0, },
		{0x32b7, 0xc0,},
		{0x32b8, 0x5, },
		{0x32b9, 0x25,},
		{0x32ba, 0x1, },
		{0x32ba, 0x51,},
		{0x32bb, 0x13,},
		{0x32bc, 0x0, },
		{0x32bc, 0x0, },
		{0x32bc, 0x0, },
		{0x32bd, 0x45,},
		{0x32be, 0x0, },
		{0x32be, 0xd0,},
		{0x32bf, 0x5, },
		{0x32c0, 0x26,},
		{0x32c1, 0x1, },
		{0x32c1, 0x61,},
		{0x32c2, 0x13,},
		{0x32c3, 0x0, },
		{0x32c3, 0x0, },
		{0x32c3, 0x0, },
		{0x32c4, 0x4e,},
		{0x32c5, 0x0, },
		{0x32c5, 0x20,},
		{0x32c6, 0x5, },
		{0x32c7, 0x2f,},
		{0x32c8, 0x1, },
		{0x32c8, 0x31,},
		{0x32c9, 0x13,},
		{0x32ca, 0x0, },
		{0x32ca, 0x0, },
		{0x32ca, 0x0, },
		{0x32cb, 0x46,},
		{0x32cc, 0x0, },
		{0x32cc, 0xa0,},
		{0x32cd, 0x4, },
		{0x32ce, 0x27,},
		{0x32cf, 0x1, },
		{0x32cf, 0xb1,},
		{0x32d0, 0x12,},
		{0x32d1, 0x0, },
		{0x32d1, 0x0, },
		{0x32d1, 0x0, },
		{0x32d2, 0x46,},
		{0x32d3, 0x0, },
		{0x32d3, 0xa0,},
		{0x32d4, 0x4, },
		{0x32d5, 0x27,},
		{0x32d6, 0x1, },
		{0x32d6, 0xb1,},
		{0x32d7, 0x12,},
		{0x32d8, 0x0, },
		{0x32d8, 0x0, },
		{0x32d8, 0x0, },
		{0x32d9, 0x54,},
		{0x32da, 0x0, },
		{0x32da, 0xa0,},
		{0x32db, 0x5, },
		{0x32dc, 0x0, },
		{0x32dc, 0x0, },
		{0x32dc, 0x0, },
		{0x32dd, 0x10,},
		{0x32de, 0x1, },
		{0x32de, 0x21,},
		{0x32df, 0x11,},
		{0x32e0, 0x0, },
		{0x32e0, 0x0, },
		{0x32e0, 0x0, },
		{0x32e1, 0x1, },
		{0x32e2, 0x0, },
		{0x32e2, 0x50,},
		{0x32e3, 0x0, },
		{0x32e4, 0x0, },
		{0x32e4, 0x0, },
		{0x32e4, 0x0, },
		{0x32e5, 0x7, },
		{0x32e6, 0x0, },
		{0x32e6, 0xb0,},
		{0x32e7, 0x0, },
		{0x32e8, 0x0, },
		{0x32e8, 0x0, },
		{0x32e8, 0x0, },
		{0x32e9, 0x78,},
		{0x32ea, 0x1, },
		{0x32ea, 0xc1,},
		{0x32eb, 0x1e,},
		{0x32ec, 0x0, },
		{0x32ec, 0x0, },
		{0x32ec, 0x0, },
		{0x32ed, 0x1e,},
		{0x32ee, 0xf, },
		{0x32ef, 0x8, },
		{0x32f0, 0x2, },
		{0x32f1, 0x2, },
		{0x32f2, 0x2, },
		{0x32f3, 0x2, },
		{0x32f4, 0xc4,},
		{0x32f5, 0x0, },
		{0x32f5, 0x40,},
		{0x32f6, 0x4, },
		{0x32f7, 0x0, },
		{0x32f7, 0x0, },
		{0x32f7, 0x0, },
		//End of file
		{0x0000, 0xCA},
};

CyU3PReturnStatus_t CyFx3_Sensor_Reg_I2c_Write_Bulk()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	uint16_t regAdd = 0;
	uint8_t regValue = 0;
	uint32_t cnt = 0;
	//	  cnt < (sizeof(sensorConfig)/(sizeof(uint16_t)+sizeof(uint8_t)))
	for (cnt = 0; ; cnt++)
	{
		regAdd = sensorConfig[cnt].sensorAddr;
		regValue = sensorConfig[cnt].sensorVal;

		//Break at the last address
		if(regAdd == 0)
		{
			status = CyFx3_Sensor_Reg_I2c_Write(0x34, 0x3000, 0);
			status = CyFx3_Sensor_Reg_I2c_Write(0x34, 0x3002, 0);
			break;
		}

		status = CyFx3_Sensor_Reg_I2c_Write(0x34, regAdd, regValue);
		if (status != CY_U3P_SUCCESS)
		{
			break;
		}
		else
		{
			CyU3PDebugPrint(4,"\r\nSensorConfig Write %x = %x",regAdd, regValue);
		}
	}
	status = CyFx3_Isp_videoControl_Reg_I2c_Write(0x0A,&autoExposure_start,&autoExposure_start.regValue);

	return status;
}

/*********************Added by HPPC**********************************************************/
CyU3PReturnStatus_t CyFx3_Isp_videoControl_Reg_I2c_Read(uint8_t slave_addr, CyIspCategoryReg_t* regPtr, uint32_t *regValue)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t cnt = 0;

	preamble.buffer[0] = slave_addr; /* Slave address: write operation */

	preamble.buffer[1] = regPtr->regAddr;
	preamble.buffer[2] = slave_addr + 1; /* Slave address: read operation */
	preamble.length = 3;
	preamble.ctrlMask = 0x0002;

	for (cnt = 0; cnt < 3; cnt++)
	{
		status = CyU3PI2cReceiveBytes(&preamble, (uint8_t*)regValue, 4, 10);
		if (status == CY_U3P_SUCCESS)
		{
			break;
		}
	}
	CyU3PDebugPrint(4,"Video Control I2C Read reg: %x = %x",regPtr->regAddr, *regValue);

	return status;
}

/*I2C Category Write function*/
CyU3PReturnStatus_t CyFx3_Isp_videoControl_Reg_I2c_Write(uint8_t slave_addr, CyIspCategoryReg_t* regPtr, uint32_t *regValue)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PI2cPreamble_t preamble;
	uint8_t cnt = 0;

	preamble.buffer[0] = slave_addr; /* Slave address: write operation */
	preamble.buffer[1] = regPtr->regAddr;
	preamble.length = 2;
	preamble.ctrlMask = 0x0000;

	for (cnt = 0; cnt < 3; cnt++)
	{
		status = CyU3PI2cTransmitBytes(&preamble, (uint8_t*)regValue, 4, 10);
		if (status == CY_U3P_SUCCESS)
		{
			CyU3PDebugPrint(4,"\r\nVideo Control I2C Write reg: %x = %x",regPtr->regAddr, *regValue);
			break;
		}
	}
	return status;
}

/********************************************************************************************/

/*Function to set AE mode (auto/manual) */
CyU3PReturnStatus_t CyFx3_Isp_AEModeControl(uint32_t *AEControl, CyBool_t isREgWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isREgWrite)
	{
		if ((*AEControl == FX3_UVC_VC_AE_AUTO_MODE_VALUE) || (*AEControl == FX3_UVC_VC_AE_MANUAL_MODE_VALUE))
		{
			if(*AEControl == FX3_UVC_VC_AE_AUTO_MODE_VALUE)
			{
				autoExposure.regValue = 2;
				status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoExposure, &autoExposure.regValue);
			}
			else
			{
				autoExposure.regValue = 0;
				status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoExposure, &autoExposure.regValue);

			}
			if(status != CY_U3P_SUCCESS)
				return status;
		}
		else
			return CY_U3P_ERROR_BAD_ARGUMENT;

	}
	else           /*On read, return value is Legacy AE control value*/
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &autoExposure, &autoExposure.regValue);
		if(autoExposure.regValue == 0)
			*AEControl = FX3_UVC_VC_AE_MANUAL_MODE_VALUE;
		else
			*AEControl = FX3_UVC_VC_AE_AUTO_MODE_VALUE;
	}

	return status;
}


/*Function to Set Manual Exposure setting*/
CyU3PReturnStatus_t CyFx3_Isp_ManualExposureControl(uint32_t *maualAETimeCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	if(isRegWrite)
	{
		autoExposure_start.regValue = 0;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoExposure, &autoExposure.regValue);
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(0x08,&autoExposure,&autoExposure.regValue);

		manualExposure.regValue = *maualAETimeCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x0A, &manualExposure, &manualExposure.regValue);
		if(status != CY_U3P_SUCCESS)
			return status;
	}
	else           /*On read, return value is Legacy AE control value*/
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x0A, &manualExposure, &manualExposure.regValue);
		*maualAETimeCtrl = manualExposure.regValue;
		if(status != CY_U3P_SUCCESS)
			return status;
	}
	return status;
}

/*Function to set Brightness*/
CyU3PReturnStatus_t CyFx3_Isp_BrightnessControl(uint32_t *brightnessControl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		brightness.regValue = *brightnessControl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x0A, &brightness, &brightness.regValue);
	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x0A, &brightness, &brightness.regValue);
		*brightnessControl = brightness.regValue;
	}
	return status;
}

/*Function to set Manual Exposure*/
CyU3PReturnStatus_t CyFx3_Isp_ManualExpCtrl(uint32_t *manExpCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		manualExposure.regValue = *manExpCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x0A, &manualExposure,&manualExposure.regValue );
	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x0A, &manualExposure, &manualExposure.regValue);
		*manExpCtrl = manualExposure.regValue;
	}
	return status;
}

/*Function to set auto white balance*/
CyU3PReturnStatus_t CyFx3_Isp_AwbModeControl(uint32_t *awbCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		autoWhiteBal.regValue = *awbCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoWhiteBal, &autoWhiteBal.regValue);


	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &autoWhiteBal, &autoWhiteBal.regValue);
		*awbCtrl = autoWhiteBal.regValue;
	}
	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_MWBTempValue (uint32_t* mwbCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{

		autoWhiteBal.regValue = 0x5;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoWhiteBal,&autoWhiteBal.regValue);
		if(status != CY_U3P_SUCCESS)
		{
			return status;
		}

		manWhiteBal.regValue = *mwbCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &manWhiteBal,&manWhiteBal.regValue);

		if(status != CY_U3P_SUCCESS)
		{
			return status;
		}


//		autoWhiteBal.regValue = 0x00;
//		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &autoWhiteBal, &autoWhiteBal.regValue);
//
//		manWhiteBalEnable.regValue = 0x01;
//		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &manWhiteBalEnable, &manWhiteBalEnable.regValue);
//
//		mwbRedChannel.regValue = *mwbCtrl;
//		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &mwbRedChannel, &mwbRedChannel.regValue);
//
//		mwbGreenChannel.regValue = *mwbCtrl;
//		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &mwbGreenChannel, &mwbGreenChannel.regValue);
//
//		mwbBlueChannel.regValue = *mwbCtrl;
//		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &mwbBlueChannel, &mwbBlueChannel.regValue);

	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &manWhiteBal, &manWhiteBal.regValue);
		*mwbCtrl = manWhiteBal.regValue;
	}
	return status;

}


/*Function to set Contrast*/
CyU3PReturnStatus_t CyFx3_Isp_ContrastControl(uint32_t *contrastCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		contrast.regValue = *contrastCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &contrast, &contrast.regValue);
	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &contrast, &contrast.regValue);
		*contrastCtrl = contrast.regValue;
	}
	return status;
}

/*Function to set Gamma Control*/
CyU3PReturnStatus_t CyFx3_Isp_GammaControl(uint32_t *gammaCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		gammaReg.regValue = *gammaCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &gammaReg, &gammaReg.regValue);
	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &gammaReg, &gammaReg.regValue);
		*gammaCtrl = gammaReg.regValue;
	}
	return status;
}


/*Function to set Saturation Control*/
CyU3PReturnStatus_t CyFx3_Isp_SaturationControl(uint16_t *saturationCtrl, CyBool_t isRegWrite)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	if(isRegWrite)
	{
		saturation.regValue = *saturationCtrl;
		status = CyFx3_Isp_videoControl_Reg_I2c_Write(SLAVE_ADDR_0x08, &saturation, &saturation.regValue);
	}
	else
	{
		status = CyFx3_Isp_videoControl_Reg_I2c_Read(SLAVE_ADDR_0x08, &saturation, &saturation.regValue);
		*saturationCtrl = saturation.regValue;
	}
	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set1920x1080_60fps()
{

	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x6C;
	sensIspI2CWriteCtrl->regData = 0x780;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x70;
	sensIspI2CWriteCtrl->regData = 0x438;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set1280x720_60fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x0000006C;
	sensIspI2CWriteCtrl->regData = 0x00000500;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x00000070;
	sensIspI2CWriteCtrl->regData = 0x000002D0;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set1280x720_15fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x68;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x0000006C;
	sensIspI2CWriteCtrl->regData = 0x00000A00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x00000070;
	sensIspI2CWriteCtrl->regData = 0x000005A0;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x02;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x74;
	sensIspI2CWriteCtrl->regData = 0x01;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set640x640_60fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x6C;
	sensIspI2CWriteCtrl->regData = 0x280;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x70;
	sensIspI2CWriteCtrl->regData = 0x280;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set640x480_60fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x6C;
	sensIspI2CWriteCtrl->regData = 0x280;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x70;
	sensIspI2CWriteCtrl->regData = 0x1E0;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set640x480_30fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x68;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x6C;
	sensIspI2CWriteCtrl->regData = 0x500;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x70;
	sensIspI2CWriteCtrl->regData = 0x3C0;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

//	sensIspI2CWriteCtrl->regAddr = 0x5C;
//	sensIspI2CWriteCtrl->regData = 0x01;
//	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x74;
	sensIspI2CWriteCtrl->regData = 0x01;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);


	return status;
}


CyU3PReturnStatus_t CyFx3_Isp_Set640x360_60fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x6C;
	sensIspI2CWriteCtrl->regData = 0x280;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x70;
	sensIspI2CWriteCtrl->regData = 0x168;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);


	return status;
}

CyU3PReturnStatus_t CyFx3_Isp_Set_StillCapture_1280x720_60fps()
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	sensIspI2CWriteCtrl->regLen = 1;
	sensIspI2CWriteCtrl->dataLen = 4;
	sensIspI2CWriteCtrl->sensIspAddr = 0x0A;

	sensIspI2CWriteCtrl->regAddr = 0x0000006C;
	sensIspI2CWriteCtrl->regData = 0x00000500;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x00000070;
	sensIspI2CWriteCtrl->regData = 0x000002D0;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	sensIspI2CWriteCtrl->regAddr = 0x5C;
	sensIspI2CWriteCtrl->regData = 0x00;
	status = CyFx3_Sensor_Isp_Reg_I2c_Write(sensIspI2CWriteCtrl);

	return status;
}


/**********************************************************************************/

