/*
 ## Cypress FX3 Core Library Header (cyu3_isp.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2014,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
 *  Created on: 30-June-2015
 *  Author: savj
 */

/* FX3 Header file for APIs implementing Image Sensor specific functions
 */
#ifndef _INCLUDED_CYU3_FX3_IMAGESENSOR_H_
#define _INCLUDED_CYU3_FX3_IMAGESENSOR_H_

#include <cyu3types.h>
#include "cyfx3_videocontrol.h"


#define FRONT_SENSOR_POWER				(0)
#define REAR_SENSOR_POWER				(1)
#define REAR_SENSOR_RESET				(2)
#define FRONT_SENSOR_RESET				(3)
#define IR_SENSOR_RESET					(4)
#define FRONT_FLASH_LED					(5)
#define REAR_FLASH_LED					(6)
#define IR_LED							(7)
#define FRONT_PRIV_LED					(8)
#define REAR_PRIV_LED					(9)

#define SLAVE_ADDR_0x08 0x08
#define SLAVE_ADDR_0x0A 0x0A

/* GPIO Interrupt low to high event */
#define ISP_INTR_LOW_TO_HIGH_EVENT       (1 << 0)

/* Select rear camera or front camera */
#define ISP_SELECT_FRONT_CAMERA          (0)
#define ISP_SELECT_REAR_CAMERA           (1)

/* ISP INTR timeout values */
#define ISP_INTR_5MS                     (5)
#define ISP_INTR_100MS                   (100)
#define ISP_INTR_500MS                   (500)
#define ISP_INTR_1000MS                  (1000)
#define ISP_INTR_2000MS                  (2000)
#define ISP_INTR_7000MS                  (7000)

/*ISP Firmware size*/
#define ISP_FIRMWARE_SIZE                (16384)

/* Address where the ISP Firmware version is stored in ISP
 * Note that actual if address is 0x18 then I2C address is 0x18000000*/
#define ISP_FW_VER_I2C_ADDR              (0x18001000)

/* slave address for ISP and sensor*/
#define ISP_SLAVE_ADDR_1                 (0x08)
#define ISP_SLAVE_ADDR_2                 (0x0A)
#define SENSOR_SLAVE_ADDR                (0x34)

/*ISP Firmware version Structure size*/
#define ISP_FW_VER_SIZE                  (0x0020)

/* ISP FW Date, Time and Version field indexes*/
#define ISP_FW_DATE_FIELD_START_INDEX    (04)
#define ISP_FW_TIME_FIELD_START_INDEX    (16)
#define ISP_FW_VER_MAJOR_FIELD_INDEX     (27)
#define ISP_FW_VER_MINOR_FIELD_INDEX     (28)

/* ISP Interrupt factor values*/
#define INTR_FACTOR_NO_INTR              (0x00)
#define INTR_FACTOR_VIDEO_STREAM         (0x01)
#define INTR_FACTOR_SINGLE_SHOT_FOCUS    (0x02)
#define INTR_FACTOR_CONTINUOUS_FOCUS     (0x10)
#define INTR_FACTOR_STILL_CAPTURE        (0x08)
#define INTR_FACTOR_STILL_SOUND          (0x80)

/* Still cature index for aspect ratios */
#define STILL_CAPTURE_ASPECT_16_9_INDEX  (1)
#define STILL_CAPTURE_ASPECT_3_2_INDEX   (2)
#define STILL_CAPTURE_ASPECT_4_3_INDEX   (3)

/* Auto focus mode select*/
#define ISP_AUTO_FOCUS_SINGLE_SHOT       (1)
#define ISP_AUTO_FOCUS_CONTINUOUS_MODE   (6)
#define ISP_AUTO_FOCUS_ROI_MODE_SELECT   (9)

/* AWB preset mode definitions*/
#define ISP_AWB_PRESET_CUSTOM_MODE       (8)

/** \file cyu3_isp.h
    \brief API definitions for configuring ISP, connected to FX3 over I2C bus.
    This file is applicable to FX3-ISP RDK.
    */
#define ISP_I2C_WRITE_ADDRESS            (0x0A) /*Write address*/
#define ISP_I2C_READ_ADDRESS             (0x3F) /*Read address*/

/******************ISP Enums and structure array*****************/
/* Enum to store ISP errors, starting at 0x80 to distinguish FX3 and ISP errors */
typedef enum CyFx3IspError
{
  //ISP_ERROR_BAD_INTR_FACTOR_READ = 0x80,
  //ISP_ERROR_BAD_LOG_LENGTH_VAL,
  //ISP_ERROR_BAD_FW_START_INTR_VAL,
  //ISP_ERROR_VC_OP_TIMEOUT,
  //ISP_ERROR_VC_OP_FAILED,
  ISP_ERROR_UNKNOWN_CODE
} CyFx3IspError_t;

/* Enum to store the frame rates */
typedef enum CyIspVideoPreviewFrameRate
{
  FRAME_RATE_3FPS = 1,
  FRAME_RATE_15FPS,
  FRAME_RATE_30FPS,
  FRAME_RATE_60FPS,
  FRAME_RATE_120FPS
} CyIspVideoPreviewFrameRate_t;

/* Enum to store the Resolution ID */
typedef enum CyIspVideoPreviewResId
{
  RES_ID_3264x2448 = 0,
  RES_ID_3264x2176,
  RES_ID_3264x1836,
  RES_ID_2592x1944,
  RES_ID_2592x1728,
  RES_ID_2592x1458,
  RES_ID_1920x1440,
  RES_ID_1920x1280,
  RES_ID_1920x1080,
  RES_ID_1280x720,
  RES_ID_1280x720_60FPS,
  RES_ID_640x640,
  RES_ID_640x480,
  RES_ID_640x360
} CyIspVideoPreviewResId_t;

/* Enum to store the number of bits to read from the Sensor*/
typedef enum CyIspMsXuSensDataBusWidth
{
  MS_XU_SENS_REG_DATA_8BITS = 1,
  MS_XU_SENS_REG_DATA_16BITS,
  MS_XU_SENS_REG_DATA_24BITS,
  MS_XU_SENS_REG_DATA_32BITS
} CyIspMsXuSensDataBusWidth_t;

/* Enum to store the ISP Category register Read API Data Byte indexes*/
typedef enum CyIspCatRegReadByteIndex
{
  ISP_CAT_REG_READ_BYTE0 = 1,
  ISP_CAT_REG_READ_BYTE1,
  ISP_CAT_REG_READ_BYTE2,
  ISP_CAT_REG_READ_BYTE3
} CyIspCatRegReadByteIndex_t;

/* Enum to store the ISP Category register Read API Data Byte indexes*/
typedef enum CyIspCatRegWriteByteIndex
{
  ISP_CAT_REG_WRITE_BYTE0 = 0,
  ISP_CAT_REG_WRITE_BYTE1,
  ISP_CAT_REG_WRITE_BYTE2,
  ISP_CAT_REG_WRITE_BYTE3
} CyIspCatRegWriteByteIndex_t;

/* Enum to store the AF operation status */
typedef enum CyIspAfOperationStatus
{
  ISP_AF_OP_STATUS_STOP = 0,
  ISP_AF_OP_STATUS_START,
  ISP_AF_OP_STATUS_DONE,
  ISP_AF_OP_STATUS_CAF_OPERATING,
  ISP_AF_OP_STATUS_CAF_PAUSED
} CyIspAfOperationStatus_t;

/* Structure to store the category Address, Register address, register value and
 * number of bytes to be transmitted*/
typedef struct CyIspCategoryReg
{
  uint8_t regAddr;
  uint32_t regValue;
  uint8_t count;
} CyIspCategoryReg_t;

typedef struct CySensorConfig
{
	uint16_t sensorAddr;
	uint8_t sensorVal;
}CySensorConfig_t;


/* Variable to check if the front and/or rear cam sensor is connected or not*/
extern CyIspCategoryReg_t glSensorConn;

/* Variable to store the ISP firmware Version structure*/
extern uint8_t glIspFwVer[];

/* ISP FW Start Intr category register */
extern CyIspCategoryReg_t glRdFwStartIntr;

/************************ISP Video Streaming Functions******************/

extern CyU3PReturnStatus_t CyFx3_Isp_Set1920x1080_60fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set1280x720_60fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set640x640_60fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set640x480_60fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set640x360_60fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set640x480_30fps();

extern CyU3PReturnStatus_t CyFx3_Isp_Set1280x720_15fps();

/************************ISP UVC Video Control APIs**************************/
/* Function to start/stop Continuous Auto focus */
extern CyU3PReturnStatus_t CyFx3_Isp_ContinuousFocusControl(uint8_t *cafControl, CyBool_t isLegacyControl, CyBool_t isRegWrite);

/* Function to set or read Manual focus*/
extern CyU3PReturnStatus_t CyFx3_Isp_ManualFocusControl(uint16_t *manualFocus, CyBool_t isLegacyControl, CyBool_t isRegWrite);

/*Function to set/read Zoom Val*/
extern CyU3PReturnStatus_t CyFx3_Isp_ZoomControl(uint16_t *zoomControl, CyBool_t isRegWrite);

/* Set the still image resolution */
extern CyU3PReturnStatus_t CyFx3_Isp_Set_Still_Resolution(uint8_t formatIndex, uint8_t stillCaptureIndex);

/* Get the still image resolution */
extern CyU3PReturnStatus_t CyFx3_Isp_Get_Still_Image();

/*Function to set Brightness*/
extern CyU3PReturnStatus_t CyFx3_Isp_BrightnessControl(uint32_t *BrightnessControl, CyBool_t isRegWrite);

/*Function to set/read Contrast value*/
extern CyU3PReturnStatus_t CyFx3_Isp_ContrastControl(uint32_t *contrastControl, CyBool_t isRegWrite);

/*Function to set/read Flicker mode*/
extern CyU3PReturnStatus_t CyFx3_Isp_FlickerModeControl(uint8_t *flickerModeControl, CyBool_t isRegWrite);

/*Function to set/read Saturation*/
extern CyU3PReturnStatus_t CyFx3_Isp_SaturationControl(uint16_t *saturatControl, CyBool_t isRegWrite);

/*Function to set/read Sharpness*/
extern CyU3PReturnStatus_t CyFx3_Isp_SharpnessControl(uint16_t *sharpnessControl, CyBool_t isRegWrite);

/* Function to get/set Backlight compensation*/
extern CyU3PReturnStatus_t CyFx3_Isp_BacklightCompensationControl(uint16_t *backlightComp, CyBool_t isRegWrite);

/* To retrieve ISP Firmware Version */
extern CyU3PReturnStatus_t CyFx3_Isp_GetFirmwareVersion();

/* I2C category read parameter function */
extern CyU3PReturnStatus_t CyFx3_Isp_CategoryReg_I2c_Read(CyIspCategoryReg_t* regPtr);

/* I2C Category Register Write function */
extern CyU3PReturnStatus_t CyFx3_Isp_CategoryReg_I2c_Write(CyIspCategoryReg_t* regPtr);

/* Function to start ISP firmware and boot up */
extern CyU3PReturnStatus_t CyFx3_Isp_Camera_Firmware_Start(CyBool_t getFwVer);

/* Function to set/reset SPI Slave select line */
extern CyU3PReturnStatus_t SetSSNLineValue(uint8_t gpio, CyBool_t gpioState);

/* Function to download ISP Micro boot code */
extern CyU3PReturnStatus_t CyFx3_ISP_Micro_Code_Init();

/* To start Display of color bar from sensor */
extern CyU3PReturnStatus_t CyFx3_Isp_DisplaySensorColourBars(CyBool_t isEnabled);

/* To start Display of color bar from ISP */
extern CyU3PReturnStatus_t CyFx3_Isp_DisplayIspColourBars(CyBool_t isEnabled);

/*Function to set AE mode (auto/manual) */
extern CyU3PReturnStatus_t CyFx3_Isp_AEModeControl(uint32_t *AEControl, CyBool_t isREgWrite);

/*Function to Set Manual Exposure setting*/
extern CyU3PReturnStatus_t CyFx3_Isp_ManualExposureControl(uint32_t *maualAETimeCtrl, CyBool_t isREgWrite);

extern CyU3PReturnStatus_t CyFx3_Isp_ManualExpCtrl(uint32_t *manExpCtrl, CyBool_t isRegWrite);

extern CyU3PReturnStatus_t CyFx3_Isp_GammaControl(uint32_t *gammaCtrl, CyBool_t isRegWrite);

/* Function to set/Reset AE Lock */
extern CyU3PReturnStatus_t CyFx3_Isp_AELockControl(uint8_t* lockControl, CyBool_t isREgWrite);

/* Function to set the AWB preset value */
extern CyU3PReturnStatus_t CyFx3_Isp_AwbPresetValue (uint8_t* awbPresetControl, CyBool_t isRegWrite);

/* Function to set/get the AWB Mode data */
extern CyU3PReturnStatus_t CyFx3_Isp_AwbModeControl (uint32_t* awbMode,  CyBool_t isREgWrite);

/* Function to set the Manual White Balance temperature value */
extern CyU3PReturnStatus_t CyFx3_Isp_MWBTempValue (uint32_t* mwbcontrol, CyBool_t isRegWrite);

/* Function to set/Reset AWB Lock */
extern CyU3PReturnStatus_t CyFx3_Isp_AwbLockControl(uint8_t* lockControl, CyBool_t isREgWrite);

/* Function to check the status of after AWB SET_CUR request */
extern CyU3PReturnStatus_t CyFx3_Isp_AwbCheckStatus();

/* Function to Check the sensor connection */
extern CyU3PReturnStatus_t CyFx3_Isp_CheckSensorConnection();

/* Function to Check the Intr factor from ISP */
extern CyU3PReturnStatus_t CyFx3_Isp_Check_Intr_Factor(uint16_t waitTime, uint8_t intrFactor);

/* Function to access the ISP logs*/
extern CyU3PReturnStatus_t CyFx3_Isp_Get_Isp_Debug_Logs();

/* Function to Clear the ISP logs */
extern CyU3PReturnStatus_t CyFx3_Isp_Clear_Isp_Debug_Logs();

/*Function to read/write reg mode*/
extern CyU3PReturnStatus_t CyFx3_Isp_MsXuVcmDacReg(uint16_t *vcmDacVal, CyBool_t isRegWrite);

/* Function to read the EEPROM data */
extern CyU3PReturnStatus_t CyFx3_Isp_MsXuEepromReadData(uint8_t *eepromReadData, uint8_t intfType);

/* Function to check the status of after AE SET_CUR request */
extern CyU3PReturnStatus_t CyFx3_Isp_AECheckStatus();

/* Check status for AF/CAF for rear camera */
extern CyU3PReturnStatus_t CyFx3_Isp_AfCheckStatus(uint8_t expectedStatus, uint16_t waitTime, CyBool_t disableAfIntr);

/*Function to toggle ISP GPIO */
extern CyU3PReturnStatus_t CyFx3_ISP_GPIO_toggle(uint8_t GPIO_Number, CyBool_t io_status);

/*Function to set ISP video Resolution*/
extern CyU3PReturnStatus_t CyFx3_Isp_Set_Resolution(uint8_t FullResID, uint8_t format, uint8_t usbmode, CyBool_t isRearCam);

/* Generic sensor/ISP I2C write function */
extern CyU3PReturnStatus_t CyFx3_Sensor_Isp_Reg_I2c_Write(xuSensIspRegRdWrData_t *sensIspI2CWriteCtrl);

/* Generic sensor/ISP I2C read function */
extern CyU3PReturnStatus_t CyFx3_Sensor_Isp_Reg_I2c_Read(xuSensIspRegRdWrData_t *ispI2CReadCtrl);

extern CyU3PReturnStatus_t CyFx3_Sensor_Reg_I2c_Write(uint8_t SensorAdd, uint16_t RegAdd, uint8_t value);

extern CyU3PReturnStatus_t CyFx3_Sensor_Reg_I2c_Write_Bulk();

extern CyU3PReturnStatus_t CyFx3_Isp_videoControl_Reg_I2c_Read(uint8_t slave_addr, CyIspCategoryReg_t* regPtr,uint32_t *regValue);

extern CyU3PReturnStatus_t CyFx3_Isp_videoControl_Reg_I2c_Write(uint8_t slave_addr, CyIspCategoryReg_t* regPtr,uint32_t *regValue);


#endif /* _INCLUDED_CYU3_FX3_IMAGESENSOR_H_ */
/*[]*/
